/*
 * Copyright (c) 1995, 1996 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * This header defines the flick_stream structure,
 * which is a data type representing a Flick marshaling/unmarshaling stream.
 * implemented on top of Fluke IPC.
 */
#ifndef _FLUKE_FLICK_STREAM_H_
#define _FLUKE_FLICK_STREAM_H_

struct flick_stream
{
	/*
	 * Pointer to the buffer to marshal data to/from.
	 * Always at least FLUKE_MIN_MSG_SIZE bytes in size.
	 */
	void			*data_buf;

	/*
	 * Total size of the data buffer above.
	 * During encoding, this is the limit
	 * that indicates when we have to grow the buffer.
	 */
	oskit_size_t		data_buf_size;

	/*
	 * Current byte offset in the data buffer for encoding/decoding.
	 */
	oskit_addr_t		data_ofs;

	/*
	 * Used for decoding only:
	 * size of the actual message received,
	 */
	oskit_addr_t		data_end_ofs;

	/*
	 * Used for decoding only:
	 * list of refs allocated from this thread's free list,
	 * into which references will be received.
	 */
	struct flick_ref	*refs_list;
	unsigned		refs_list_length;

	/*
	 * Array of pointers to refs to send or receive.
	 */
	struct fluke_ref	**refs;
	unsigned		refs_length;
};

#endif /* _FLUKE_FLICK_STREAM_H_ */
