/*
 * Copyright (c) 1995 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_MUTEX_H_
#define _FLUKE_MUTEX_H_

#include <oskit/machine/types.h>
#include <fluke/ob.h>

FLUKE_OB_DEFINE(fluke_mutex, FLUKE_MUTEX_SIZE);

struct fluke_mutex_state
{
	fluke_hash_t	hash;
	oskit_u32_t	flags;		/* attributes of the mutex */
	oskit_bool_t	locked;		/* TRUE if mutex is locked */

	/* Leave at least one pad word for owner.  */
};
typedef struct fluke_mutex_state fluke_mutex_state_t;

/* flag values - XXX these aren't used currently */
#define MUTEX_INHERIT_PRI	1
#define MUTEX_PRI_CEILING	2
#define MUTEX_MULTITASK		4

struct fluke_ref;

fluke_error_t fluke_mutex_create(
	fluke_mutex_t		*mutex);

fluke_error_t fluke_mutex_create_hash(
	fluke_mutex_t		*mutex,
	fluke_hash_t		*hash);

void fluke_mutex_destroy(
	fluke_mutex_t		*mutex);

void fluke_mutex_lock(
	fluke_mutex_t		*mutex);

int fluke_mutex_trylock(
	fluke_mutex_t		*mutex);

void fluke_mutex_unlock(
	fluke_mutex_t		*mutex);

void fluke_mutex_reference(
	fluke_mutex_t		*mutex,
	struct fluke_ref	*out_mutex_ref);

void fluke_mutex_move(
	fluke_mutex_t		*old_va,
	fluke_mutex_t		*new_va);

void fluke_mutex_get_state(
	fluke_mutex_t		*mutex,
	fluke_mutex_state_t	*out_state,
	struct fluke_ref	*out_owner_ref);

void fluke_mutex_set_state(
	fluke_mutex_t		*mutex,
	fluke_mutex_state_t	*state,
	struct fluke_ref	*owner_ref);


#include <fluke/machine/mutex.h>

#endif /* _FLUKE_MUTEX_H_ */
