/*
 * Copyright (c) 1996 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 *	Pthreads on top of Fluke
 */

#ifndef _FLUKE_SAC_PTHREAD_H
#define	_FLUKE_SAC_PTHREAD_H

/* XXX major namespace pollution here */
#include <fluke/ob.h>
#include <fluke/ref.h>
#include <fluke/thread.h>
#include <fluke/mutex.h> 
#include <fluke/cond.h> 
#include <fluke/pset.h>
#include <fluke/task.h>
#include <fluke/port.h> 

#include <stddef.h>
#include <sys/types.h>
#include <oskit/c/malloc.h>

/* XXX */
/*
 *	find next power of two
 */
static inline unsigned
NP2(int x) {
	signed int n=x, b=sizeof(int)*8;
	while (n > 0)
		b--, n <<= 1;
	return (1<<b);
}

#define FLUKE_ALLOC(type) \
	((type *)smemalign(NP2(sizeof(type)), sizeof(type)))
#define FLUKE_FREE(ptr, type) \
	sfree(ptr, sizeof(type))

#define	OK 	0

#include <fluke/sac/pthread/mutex.h>
#include <fluke/sac/pthread/cond.h>
#include <fluke/sac/pthread/pthread_attr.h>

typedef enum pthread_states {
	PTHREAD_ALIVE,		/* for now, this just mean not terminated */
	PTHREAD_TERMINATED
} pthread_states;

/* 
 *	a pthread struct embeds the fluke_thread structure
 *	Structure definition is NOT publicly accessable
 */
struct pthread;
typedef struct pthread *pthread_t;

void	pthread_init	(void);
int	pthread_create	(pthread_t *, 
			const pthread_attr_t *, 
			void * (*start_routine)(void *), void *);
void	pthread_exit	(void *) OSKIT_NORETURN;
pthread_t pthread_self	(void);
int	pthread_equal	(pthread_t, pthread_t);
int	pthread_join	(pthread_t, void **);
int	pthread_detach	(pthread_t);
void	pthread_yield	(void);

int	pthread_setschedparam	(pthread_t pthread, 
		int policy, struct sched_param * param);
int	pthread_getschedparam	(pthread_t pthread, 
		int * policy, struct sched_param * param);

int	pthread_kill	(struct pthread *, int);
int	pthread_signal	(int, void (*)(int));


/* 
 * Posix says this needs to be at least 128.
 */
#define PTHREAD_KEYS_MAX 16

typedef unsigned pthread_key_t;

int pthread_key_create(pthread_key_t *key, void (*destructor)(void *));
int pthread_key_delete(pthread_key_t key);
int pthread_setspecific(pthread_key_t key, const void *value);
void * pthread_getspecific(pthread_key_t key);

typedef void (*pthread_destructor_t)(void *);


#include <errno.h>
#endif /* _FLUKE_SAC_PTHREAD_H */
