/*
 * Copyright (c) 1996, 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_X86_MAPPING_H_
#define _FLUKE_X86_MAPPING_H_

#if defined(__GNUC__) && defined(FLUKE_INLINING)

#include <oskit/compiler.h>
#ifndef FLUKE_ENTRY_INLINE
#define FLUKE_ENTRY_INLINE OSKIT_INLINE
#endif
#ifdef GPROF
#define e(name) extern char _fluke_##name##_[];
#include <fluke/x86/syscall_list.h>
#undef e
#include <oskit/x86/proc_reg.h>
#include <fluke/profile.h>
#endif

FLUKE_ENTRY_INLINE fluke_error_t fluke_mapping_create(
	fluke_mapping_t	*mapping)
{
	fluke_error_t rc;

#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_mapping_create_hash_);
#endif
	asm volatile("call _fluke_mapping_create_hash"
		: "=a" (rc) : "a" (mapping), "d" (0) : "ecx", "edx");

	return rc;
}

FLUKE_ENTRY_INLINE fluke_error_t fluke_mapping_create_hash(
	fluke_mapping_t	*mapping,
	fluke_hash_t	*hashp)
{
	fluke_error_t rc;
	fluke_hash_t hash;

#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_mapping_create_hash_);
#endif
	hash = hashp ? *hashp : 0;

	asm volatile("call _fluke_mapping_create_hash"
		: "=a" (rc), "=d" (hash)
		: "a" (mapping), "d" (hash)
		: "ecx");
	if (hashp) *hashp = hash;

	return rc;
}

FLUKE_ENTRY_INLINE void fluke_mapping_destroy(
	fluke_mapping_t	*mapping)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_mapping_destroy_);
#endif
	asm volatile("call _fluke_mapping_destroy"
		: : "a" (mapping) : "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_mapping_move(
	fluke_mapping_t	*omapping,
	fluke_mapping_t	*nmapping)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_mapping_move_);
#endif
	asm volatile("call _fluke_mapping_move"
		: : "a" (omapping), "d" (nmapping) : "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_mapping_protect(
	fluke_mapping_t	*mapping,
	fluke_prot_t	prot)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_mapping_protect_);
#endif
	asm volatile("call _fluke_mapping_protect"
		: : "a" (mapping), "d" (prot) : "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_mapping_get_state(
	fluke_mapping_t			*mapping,
	struct fluke_mapping_state	*out_state,
	struct fluke_ref		*out_region_ref,
	struct fluke_ref		*out_task_ref)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_mapping_get_state_);
#endif
	asm volatile("call _fluke_mapping_get_state"
		: : "a" (mapping), "d" (out_state),
		    "c" (out_region_ref), "b" (out_task_ref)
		: "eax", "ebx", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_mapping_set_state(
	fluke_mapping_t			*mapping,
	struct fluke_mapping_state	*state,
	struct fluke_ref		*region_ref,
	struct fluke_ref		*task_ref)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_mapping_set_state_);
#endif
	asm volatile("call _fluke_mapping_set_state"
		: : "a" (mapping), "d" (state),
		    "c" (region_ref), "b" (task_ref)
		: "eax", "ebx", "ecx", "edx");
}

#endif /* __GNUC__ */

#endif /* _FLUKE_X86_MAPPING_H_ */
