/*
 * Copyright (c) 1996, 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_X86_MUTEX_H_
#define _FLUKE_X86_MUTEX_H_

#if defined(__GNUC__) && defined(FLUKE_INLINING)

#include <oskit/compiler.h>
#ifndef FLUKE_ENTRY_INLINE
#define FLUKE_ENTRY_INLINE OSKIT_INLINE
#endif
#ifdef GPROF
#define e(name) extern char _fluke_##name##_[];
#include <fluke/x86/syscall_list.h>
#undef e
#include <oskit/x86/proc_reg.h>
#include <fluke/profile.h>
#endif

FLUKE_ENTRY_INLINE fluke_error_t fluke_mutex_create(
	fluke_mutex_t		*mutex)
{
	fluke_error_t rc;

#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_mutex_create_hash_);
#endif
	asm volatile("call	_fluke_mutex_create_hash"
		: "=a" (rc) : "S" (mutex), "d" (0) : "eax", "ecx", "edx");

	return rc;
}

FLUKE_ENTRY_INLINE fluke_error_t fluke_mutex_create_hash(
	fluke_mutex_t	*mutex,
	fluke_hash_t	*hashp)
{
	fluke_error_t rc;
	fluke_hash_t hash;

	hash = hashp ? *hashp : 0;
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_mutex_create_hash_);
#endif
	asm volatile("call _fluke_mutex_create_hash"
		: "=a" (rc), "=d" (hash)
		: "S" (mutex), "d" (hash)
		: "ecx");
	if (hashp) *hashp = hash;

	return rc;
}

FLUKE_ENTRY_INLINE void fluke_mutex_destroy(
	fluke_mutex_t		*mutex)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_mutex_destroy_);
#endif
	asm volatile("call	_fluke_mutex_destroy"
		: : "S" (mutex) : "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_mutex_lock(
	fluke_mutex_t		*mutex)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_mutex_lock_);
#endif
	asm volatile("call	_fluke_mutex_lock"
		: : "S" (mutex) : "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE int fluke_mutex_trylock(
	fluke_mutex_t		*mutex)
{
	int rc;

#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_mutex_trylock_);
#endif
	asm volatile("call	_fluke_mutex_trylock"
		: "=a" (rc) : "S" (mutex) : "ecx", "edx");

	return rc;
}

FLUKE_ENTRY_INLINE void fluke_mutex_unlock(
	fluke_mutex_t		*mutex)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_mutex_unlock_);
#endif
	asm volatile("call	_fluke_mutex_unlock"
		: : "S" (mutex) : "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_mutex_reference(
	fluke_mutex_t		*mutex,
	struct fluke_ref	*out_mutex_ref)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_mutex_reference_);
#endif
	asm volatile("call	_fluke_mutex_reference"
		: : "a" (mutex), "d" (out_mutex_ref) : "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_mutex_move(
	fluke_mutex_t		*old_va,
	fluke_mutex_t		*new_va)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_mutex_move_);
#endif
	asm volatile("call	_fluke_mutex_move"
		: : "a" (old_va), "d" (new_va) : "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_mutex_get_state(
	fluke_mutex_t		*mutex,
	fluke_mutex_state_t	*out_state,
	struct fluke_ref	*out_owner_ref)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_mutex_get_state_);
#endif
	asm volatile("call	_fluke_mutex_get_state"
		: : "a" (mutex), "d" (out_state), "c" (out_owner_ref)
		: "eax", "ecx", "edx");
}

FLUKE_ENTRY_INLINE void fluke_mutex_set_state(
	fluke_mutex_t		*mutex,
	fluke_mutex_state_t	*state,
	struct fluke_ref	*owner_ref)
{
#ifdef GPROF
	_mcount(get_eip(), (oskit_addr_t)_fluke_mutex_set_state_);
#endif
	asm volatile("call	_fluke_mutex_set_state"
		: : "a" (mutex), "d" (state), "c" (owner_ref)
		: "eax", "ecx", "edx");
}

#endif /* __GNUC__ */

#endif /* _FLUKE_X86_MUTEX_H_ */
