/*
 * Copyright (c) 1995, 1996, 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * This file defines the magic Fluke entrypoints for all system calls,
 * as well as the machine registers used for each parameter
 * in each Fluke system call.
 */
#ifndef _FLUKE_X86_SYSCALL_H_
#define _FLUKE_X86_SYSCALL_H_

/* Misc */
/* XXX This should probably not be a syscall */
#define fluke_doprof__entry					0x04000000
#define fluke_doprof__foo					eax

#define fluke_profil__entry					0x040000a0
#define fluke_profil__offset					eax
#define fluke_profil__scale					ebx
#define fluke_profil__samples					ecx
#define fluke_profil__size					edx

#define fluke_debug_dump_flob__entry                            0x04000020
#define fluke_debug_dump_flob__sob                              eax

#define fluke_debug_puts__entry                                 0x04000010
#define fluke_debug_puts__str                                   eax
#define fluke_debug_puts__len                                   ebx

#define fluke_debug_label_flob__entry                           0x04000030
#define fluke_debug_label_flob__sob                             eax
#define fluke_debug_label_flob__str                             ebx

#define fluke_debug_dump_memuse__entry                          0x04001050

#define fluke_debug_null_syscall__entry                         0x04001060

#define fluke_debug_set_monregs__entry                          0x04001070
#define fluke_debug_set_monregs__reg0                           eax
#define fluke_debug_set_monregs__reg1                           ebx

#define fluke_instrument_thread__entry                          0x04000e00 /* XXX */
#define fluke_instrument_thread__thread                         eax
#define fluke_instrument_thread__op                             ebx

#define fluke_nop__entry					0x04000ff0

#define fluke_memcpy__entry				        0x04000??0
#define fluke_memcpy__src_va					esi
#define fluke_memcpy__dest_va					edi
#define fluke_memcpy__size					ecx

#define fluke_memset__entry					0x04000??0
#define fluke_memset__va					edi
#define fluke_memset__size					ecx

/*** Thread ***/

#define fluke_thread_create_hash__entry				0x04000330
#define fluke_thread_create_hash__thread			eax
#define fluke_thread_create_hash__hash				edx
#define fluke_thread_create_hash__rc				eax

#define fluke_thread_destroy__entry				0x04000370
#define fluke_thread_destroy__thread				eax

#define fluke_thread_set_state__entry				0x040003c0
#if FLASK
#define fluke_thread_set_state_secure__entry			0x040003d0
#endif

#define fluke_thread_get_state__entry				0x040003e0
#if FLASK
#define fluke_thread_get_state_secure__entry			0x040003f0
#endif

#define fluke_thread_enable_exceptions__entry			0x04000400

#define fluke_thread_disable_exceptions__entry			0x04000440

#define fluke_thread_interrupt__entry				0x04000490
#define fluke_thread_interrupt__thread				eax

#define fluke_thread_self__entry				0x040004e0
#define fluke_thread_self__thread				eax

#define fluke_thread_move__entry				0x04000510
#define fluke_thread_move__oaddr				eax
#define fluke_thread_move__naddr				edx

#define fluke_thread_reference__entry				0x04000530
#define fluke_thread_reference__thread				eax
#define fluke_thread_reference__ref				edx

#define fluke_thread_get_saved_state__entry			0x04000040
#define fluke_thread_get_saved_state__cur_thread		eax
#define fluke_thread_get_saved_state__orig_eax			esi
#define fluke_thread_get_saved_state__orig_ecx			ecx
#define fluke_thread_get_saved_state__orig_edx			edx
#define fluke_thread_get_saved_state__orig_eip			edi
#define fluke_thread_get_saved_state__orig_eflags		ebx

#define fluke_thread_set_saved_state__entry			0x040000c0
#define fluke_thread_set_saved_state__cur_thread		eax
#define fluke_thread_set_saved_state__orig_eax			esi
#define fluke_thread_set_saved_state__orig_ecx			ecx
#define fluke_thread_set_saved_state__orig_edx			edx
#define fluke_thread_set_saved_state__orig_eip			edi
#define fluke_thread_set_saved_state__orig_eflags		ebx

#define fluke_thread_return_from_exception__entry		0x04000140
#define fluke_thread_return_from_exception__cur_thread		eax

#define fluke_thread_set_handlers__entry			0x040001a0
#define fluke_thread_set_handlers__cur_thread			eax
#define fluke_thread_set_handlers__trap_handler			ecx
#define fluke_thread_set_handlers__interrupt_handler		edx
#define fluke_thread_set_handlers__client_alert_handler		esi
#define fluke_thread_set_handlers__server_alert_handler		edi

#define fluke_thread_get_handlers__entry			0x04000200
#define fluke_thread_get_handlers__cur_thread			eax
#define fluke_thread_get_handlers__trap_handler			ecx
#define fluke_thread_get_handlers__interrupt_handler		edx
#define fluke_thread_get_handlers__client_alert_handler		esi
#define fluke_thread_get_handlers__server_alert_handler		edi

#define fluke_thread_get_client__entry				0x04000280
#if FLASK
#define fluke_thread_get_client_secure__entry			0x04000290
#endif
#define fluke_thread_get_client__cur_thread			eax
#define fluke_thread_get_client__client				edx
#define fluke_thread_get_client__flags				ecx
#if FLASK
#define fluke_thread_get_client__essid				esi
#define fluke_thread_get_client__cecsid				edi
#endif

#define fluke_thread_set_client__entry				0x040002a0
#if FLASK
#define fluke_thread_set_client_secure__entry			0x040002b0
#endif
#define fluke_thread_set_client__cur_thread			eax
#define fluke_thread_set_client__client				edx
#define fluke_thread_set_client__flags				ecx
#if FLASK
#define fluke_thread_set_client__essid				esi
#define fluke_thread_set_client__cecsid				edi
#endif

#define fluke_thread_get_server__entry				0x040002c0
#if FLASK
#define fluke_thread_get_server_secure__entry			0x040002d0
#endif
#define fluke_thread_get_server__cur_thread			eax
#define fluke_thread_get_server__server				edx
#define fluke_thread_get_server__flags				ecx
#if FLASK
#define fluke_thread_get_server__ecsid				esi
#define fluke_thread_get_server__sessid				edi
#endif

#define fluke_thread_set_server__entry				0x040002e0
#if FLASK
#define fluke_thread_set_server_secure__entry			0x040002f0
#endif
#define fluke_thread_set_server__cur_thread			eax
#define fluke_thread_set_server__server				edx
#define fluke_thread_set_server__flags				ecx
#if FLASK
#define fluke_thread_set_server__ecsid				esi
#define fluke_thread_set_server__sessid				edi
#endif

#define fluke_thread_yield__entry				0x04000c00
#define fluke_thread_yield__cur_thread				eax
#define fluke_thread_yield__target				edx

/*** Task ***/

#define fluke_task_create_hash__entry				0x04000620
#define fluke_task_create_hash__task				eax
#define fluke_task_create_hash__hash				edx
#define fluke_task_create_hash__rc				eax

#define fluke_task_destroy__entry				0x04000660
#define fluke_task_destroy__task				eax

#define fluke_task_set_state__entry				0x040006a0
#define fluke_task_set_state__task				eax
#define fluke_task_set_state__state				edx
#define fluke_task_set_state__keeper				ecx

#define fluke_task_get_state__entry				0x040006c0
#define fluke_task_get_state__task				eax
#define fluke_task_get_state__state				edx
#define fluke_task_get_state__keeper				ecx

#define fluke_task_move__entry					0x040006e0
#define fluke_task_move__oaddr					eax
#define fluke_task_move__naddr					edx

#define fluke_task_reference__entry				0x04000710
#define fluke_task_reference__task				eax
#define fluke_task_reference__ref				edx


/*** Mapping ***/

#define fluke_mapping_create_hash__entry			0x04000820
#define fluke_mapping_create_hash__mapping			eax
#define fluke_mapping_create_hash__hash				edx
#define fluke_mapping_create_hash__rc				eax

#define fluke_mapping_destroy__entry				0x04000860
#define fluke_mapping_destroy__mapping				eax

#define fluke_mapping_set_state__entry				0x040008a0
#define fluke_mapping_set_state__mapping			eax
#define fluke_mapping_set_state__state				edx
#define fluke_mapping_set_state__region				ecx
#define fluke_mapping_set_state__task				ebx

#define fluke_mapping_get_state__entry				0x040008d0
#define fluke_mapping_get_state__mapping			eax
#define fluke_mapping_get_state__state				edx
#define fluke_mapping_get_state__region				ecx
#define fluke_mapping_get_state__task				ebx

#define fluke_mapping_move__entry				0x040008f0
#define fluke_mapping_move__oaddr				eax
#define fluke_mapping_move__naddr				edx

#define fluke_mapping_protect__entry				0x04000910
#define fluke_mapping_protect__mapping				eax
#define fluke_mapping_protect__prot				edx


/*** Region ***/

#define fluke_region_create_hash__entry				0x04000a40
#define fluke_region_create_hash__region			eax
#define fluke_region_create_hash__hash				edx
#define fluke_region_create_hash__rc				eax

#define fluke_region_destroy__entry				0x04000a80
#define fluke_region_destroy__region				eax

#define fluke_region_set_state__entry				0x04000ac0
#define fluke_region_set_state__region				eax
#define fluke_region_set_state__state				edx
#define fluke_region_set_state__task				ecx
#define fluke_region_set_state__keeper				ebx

#define fluke_region_get_state__entry				0x04000ae0
#define fluke_region_get_state__region				eax
#define fluke_region_get_state__state				edx
#define fluke_region_get_state__task				ecx
#define fluke_region_get_state__keeper				ebx

#define fluke_region_move__entry				0x04000b10
#define fluke_region_move__oaddr				eax
#define fluke_region_move__naddr				edx

#define fluke_region_reference__entry				0x04000b30
#define fluke_region_reference__region				eax
#define fluke_region_reference__ref				edx

#define fluke_region_protect__entry				0x04000b50
#define fluke_region_protect__region				eax
#define fluke_region_protect__prot				edx

#define fluke_region_search__entry				0x04000b90
#define fluke_region_search__region				eax
#define fluke_region_search__offset				edx
#define fluke_region_search__size				ecx
#define fluke_region_search__obs				ebx
#define fluke_region_search__count				esi


/*** Mutex ***/

#define fluke_mutex_create_hash__entry				0x04001600
#define fluke_mutex_create_hash__mutex				esi
#define fluke_mutex_create_hash__hash				edx
#define fluke_mutex_create_hash__rc				eax

#define fluke_mutex_destroy__entry				0x04001740
#define fluke_mutex_destroy__mutex				esi

#define fluke_mutex_get_state__entry				0x04001770
#define fluke_mutex_get_state__mutex				eax
#define fluke_mutex_get_state__state				edx
#define fluke_mutex_get_state__owner				ecx

#define fluke_mutex_set_state__entry				0x040017b0
#define fluke_mutex_set_state__mutex				eax
#define fluke_mutex_set_state__state				edx
#define fluke_mutex_set_state__owner				ecx

#define fluke_mutex_lock__entry					0x04001680
#define fluke_mutex_lock__mutex					esi

#define fluke_mutex_unlock__entry				0x04001700
#define fluke_mutex_unlock__mutex				esi

#define fluke_mutex_trylock__entry				0x04001640
#define fluke_mutex_trylock__mutex				esi
#define fluke_mutex_trylock__rc					eax

#define fluke_mutex_move__entry					0x04001790
#define fluke_mutex_move__oaddr					eax
#define fluke_mutex_move__naddr					edx

#define fluke_mutex_reference__entry				0x040017e0
#define fluke_mutex_reference__mutex				eax
#define fluke_mutex_reference__ref				edx



/*** Cond ***/

#define fluke_cond_create_hash__entry				0x04001920
#define fluke_cond_create_hash__cond				edi
#define fluke_cond_create_hash__hash				edx
#define fluke_cond_create_hash__rc				eax

#define fluke_cond_destroy__entry				0x04001960
#define fluke_cond_destroy__cond				edi

#define fluke_cond_get_state__entry				0x04001990
#define fluke_cond_get_state__cond				eax
#define fluke_cond_get_state__state				edx
#define fluke_cond_get_state__inheritor				ecx

#define fluke_cond_set_state__entry				0x040019b0
#define fluke_cond_set_state__cond				eax
#define fluke_cond_set_state__state				edx
#define fluke_cond_set_state__inheritor				ecx

#define fluke_cond_wait__entry					0x04001800
#define fluke_cond_wait__cond					edi
#define fluke_cond_wait__mutex					esi

#define fluke_cond_signal__entry				0x04001880
#define fluke_cond_signal__cond					edi

#define fluke_cond_broadcast__entry				0x040018c0
#define fluke_cond_broadcast__cond				edi

#define fluke_cond_move__entry					0x040019d0
#define fluke_cond_move__oaddr					eax
#define fluke_cond_move__naddr					edx

#define fluke_cond_reference__entry				0x040019f0
#define fluke_cond_reference__cond				eax
#define fluke_cond_reference__ref				edx


/*** Port ***/

#define fluke_port_create_hash__entry				0x04001a10
#define fluke_port_create_hash__port				eax
#define fluke_port_create_hash__hash				edx
#define fluke_port_create_hash__rc				eax

#define fluke_port_destroy__entry				0x04001a50
#define fluke_port_destroy__port				eax

#define fluke_port_set_state__entry				0x04001a90
#define fluke_port_set_state__port				eax
#define fluke_port_set_state__state				edx
#define fluke_port_set_state__pset				ecx

#define fluke_port_get_state__entry				0x04001ab0
#define fluke_port_get_state__port				eax
#define fluke_port_get_state__state				edx
#define fluke_port_get_state__pset				ecx

#define fluke_port_move__entry					0x04001ad0
#define fluke_port_move__oaddr					eax
#define fluke_port_move__naddr					edx

#define fluke_port_reference__entry				0x04001af0
#define fluke_port_reference__port				eax
#define fluke_port_reference__ref				edx


/*** Port Set ***/

#define fluke_pset_create_hash__entry				0x04001c30
#define fluke_pset_create_hash__pset				eax
#define fluke_pset_create_hash__hash				edx
#define fluke_pset_create_hash__rc				eax

#define fluke_pset_destroy__entry				0x04001c70
#define fluke_pset_destroy__pset				eax

#define fluke_pset_set_state__entry				0x04001d30
#define fluke_pset_set_state__pset				eax
#define fluke_pset_set_state__state				edx

#define fluke_pset_get_state__entry				0x04001d60
#define fluke_pset_get_state__pset				eax
#define fluke_pset_get_state__state				edx

#define fluke_pset_move__entry					0x04001cb0
#define fluke_pset_move__oaddr					eax
#define fluke_pset_move__naddr					edx

#define fluke_pset_reference__entry				0x04001cd0
#define fluke_pset_reference__pset				eax
#define fluke_pset_reference__ref				edx


/*** IPC ***/

/* Standard register conventions common to almost all IPC syscalls */
#define fluke_ipc__dest						eax
#define fluke_ipc__counts					eax
#define fluke_ipc__params					ebx
#define fluke_ipc__port_alias					ebx
#define fluke_ipc__send_buf_words				ecx
#define fluke_ipc__send_ref_tab					edx
#define fluke_ipc__send_buf_buf					esi
#define fluke_ipc__send_buf_tab					edi
#define fluke_ipc__recv_buf_words				ecx
#define fluke_ipc__recv_ref_tab					edx
#define fluke_ipc__recv_buf_tab					esi
#define fluke_ipc__recv_buf_buf					edi
#define fluke_ipc__recv_min_msg_0				esi
#define fluke_ipc__recv_min_msg_1				edi
#if FLASK
/* only if _secure interface is used */
#define fluke_ipc__client_sid					edx
#define fluke_ipc__out_client_sid				esi
#endif

/* IPC syscalls that use the above register conventions */
#define fluke_ipc_send__entry					0x04002080
#if FLASK
#define fluke_ipc_send_secure__entry				0x04002090
#endif
#define fluke_ipc_call__entry					0x04002100
#if FLASK
#define fluke_ipc_call_secure__entry				0x04002110
#endif
#define fluke_ipc_reply__entry					0x04002180
#define fluke_ipc_reply_wait_receive__entry			0x040021c0
#if FLASK
#define fluke_ipc_reply_wait_receive_secure__entry		0x040021d0
#endif
#define fluke_ipc_client_connect_send__entry			0x040021e0
#if FLASK
#define fluke_ipc_client_connect_send_secure__entry		0x040021f0
#endif
#define fluke_ipc_client_ack_send__entry			0x04002560
#define fluke_ipc_client_send__entry				0x04002380
#define fluke_ipc_client_connect_send_over_receive__entry	0x04002500
#if FLASK
#define fluke_ipc_client_connect_send_over_receive_secure__entry 0x04002520
#endif
#define fluke_ipc_client_ack_send_over_receive__entry		0x04002420
#define fluke_ipc_client_send_over_receive__entry		0x04002480
#define fluke_ipc_client_receive__entry				0x04002300
#define fluke_ipc_client_disconnect__entry			0x04002220
#define fluke_ipc_server_ack_send__entry			0x04002da0
#define fluke_ipc_server_send__entry				0x04002b80
#define fluke_ipc_server_ack_send_wait_receive__entry		0x04002d00
#if FLASK
#define fluke_ipc_server_ack_send_wait_receive_secure__entry	0x04002d10
#endif
#define fluke_ipc_server_ack_send_over_receive__entry		0x04002c20
#define fluke_ipc_server_send_wait_receive__entry		0x04002cc0
#if FLASK
#define fluke_ipc_server_send_wait_receive_secure__entry	0x04002ce0
#endif
#define fluke_ipc_server_send_over_receive__entry		0x04002c80
#define fluke_ipc_server_receive__entry				0x04002b00
#define fluke_ipc_server_disconnect__entry			0x04002a20
#define fluke_ipc_wait_receive__entry				0x040029e0
#if FLASK
#define fluke_ipc_wait_receive_secure__entry			0x04002a00
#endif

#define fluke_ipc_setup__entry					0x04002ee0
#define fluke_ipc_setup__pset					eax
#define fluke_ipc_setup__esp					ecx
#define fluke_ipc_setup__oneway_eip				edx
#define fluke_ipc_setup__idempotent_eip				esi
#define fluke_ipc_setup__reliable_eip				edi


/*** Ref ***/

#define fluke_ref_create__entry					0x04002f00
#define fluke_ref_create__ref					eax
#define fluke_ref_create__hash					edx
#define fluke_ref_create__rc					eax

#define fluke_ref_destroy__entry				0x04002f30
#define fluke_ref_destroy__ref					eax

#define fluke_ref_type__entry					0x04002f80
#define fluke_ref_type__ref					eax
#define fluke_ref_type__result_type				eax

#define fluke_ref_hash__entry					0x04002f60
#define fluke_ref_hash__ref					eax
#define fluke_ref_hash__result_hash				eax

#if FLASK
#define fluke_ref_sid__entry					0x04002f70
#define fluke_ref_sid__ref					eax
#define fluke_ref_sid__result_sid				eax
#endif

#define fluke_ref_check__entry					0x04002f90
#define fluke_ref_check__ref					eax
#define fluke_ref_check__result					eax

#define fluke_ref_compare__entry				0x04002fb0
#define fluke_ref_compare__ref1					eax
#define fluke_ref_compare__ref2					edx
#define fluke_ref_compare__result				eax

#define fluke_ref_copy__entry					0x04002fc0
#define fluke_ref_copy__fref					eax
#define fluke_ref_copy__tref					edx

#define fluke_ref_move__entry					0x04002fd0
#define fluke_ref_move__oaddr					eax
#define fluke_ref_move__naddr					edx


/*** Security ***/

#if FLASK
#define fluke_security_reference__entry				0x04002ec0
#define fluke_security_reference__ref				eax
#endif

#endif /* _FLUKE_X86_SYSCALL_H_ */
