/*
 * Copyright (c) 1997-1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * Contributed by the Computer Security Research division,
 * INFOSEC Research and Technology Office, NSA.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#if FLASK

#ifndef _FLUKE_KERNEL_AVC_IPC_H_
#define _FLUKE_KERNEL_AVC_IPC_H_

#include "s_ref.h"
#include "avc.h"
#include <fluke/machine/syscall.h>


OSKIT_INLINE int
avc_has_permission_impersonate(security_id_t sid,
			       security_id_t esid,
			       access_vector_t requested,
			       struct s_thread * sth,
			       int status)
{
    struct avc_entry *ae;
    unsigned        aever;
    access_vector_t allowed;
    int             rc;

#if AVC_OPTIMIZE_SID_EQUALITY
    if (sid == esid)
	return 0;
#endif

    rc = avc_compute_access_vector(SECCLASS_SUBJECT, sid,
				   SECCLASS_SUBJECT, esid, requested,
				   &allowed, &ae, &aever);
    if (rc)
	return (rc);

    if ((requested & allowed) != requested) {
	IPC_STATUS(sth) = status;
	EXC_SET_IP(sth, fluke_nop__entry);
	return KR_RESTART;
    }
    return 0;
}


OSKIT_INLINE int
avc_has_permission_ipc(struct s_ref * ref,
		       security_id_t ecsid,
		       security_id_t essid,
		       access_vector_t requested,
		       struct s_thread * sth,
		       int status)
{
    struct avc_entry *ae = ref->ae;
    access_vector_t allowed;
    int             rc;

#if AVC_OPTIMIZE_SID_EQUALITY
    if (ecsid == essid)
	return 0;
#endif

#if AVC_KEEP_STATS
    avc_cache_stats[AVC_ENTRY_LOOKUPS]++;
#endif

    if (ae) {
	mutex_lock(ae->lock);
	if (ae->version == ref->aever && ae->ssid == ecsid &&
	    ((ae->decided & requested) == requested)) {
#if AVC_KEEP_STATS
	    avc_cache_stats[AVC_ENTRY_HITS]++;
#endif
	    ae->used = TRUE;
	    allowed = ae->allowed;
	    mutex_unlock(ae->lock);
	} else {
	    mutex_unlock(ae->lock);
	    ae = 0;
	}
    }
    if (!ae) {
#if AVC_KEEP_STATS
	avc_cache_stats[AVC_ENTRY_MISSES]++;
#endif
	rc = avc_compute_access_vector(SECCLASS_SUBJECT, ecsid,
				       SECCLASS_SUBJECT, essid, requested,
				       &allowed, &(ref->ae), &(ref->aever));
	if (rc) {
	    s_ref_unlock(ref);
	    return rc;
	}
    }
    s_ref_unlock(ref);
    if ((requested & allowed) != requested) {
	IPC_STATUS(sth) = status;
	EXC_SET_IP(sth, fluke_nop__entry);
	return KR_RESTART;
    }
    return 0;
}


#endif				/* _FLUKE_KERNEL_AVC_IPC_H_ */

#endif
