/*
 * Copyright (c) 1997 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * Interface to the currency hierarchy.
 */

#ifndef _FLUKE_STRIDE_CURRENCY_H_
#define _FLUKE_STRIDE_CURRENCY_H_

#include "list.h"

struct s_thread;

typedef struct currency_state {

	/* The thread is in the ready queue */
	int active; 

	/* Number of tickets in its own currency */
	unsigned own_tickets;

	/* Total number of tickets emitted in this currency */
	unsigned total_tickets;

	/* Number of tickets that receives from its father */
	unsigned inherit_tickets;

	/* Number of base tickets inherited. It is used to avoid
	   recalculating from the root we modify a branch.*/
	unsigned base_tickets;

	/* List of threads that recive CPU from this thread */
	list_t child_list;
  
	/* Parent thread */
	struct s_thread *parent;

	/* Currency Scaling Factor. XXX to be implemented */
	unsigned scaling_factor;

} currency_state_t;


typedef struct currency {
	struct s_thread *root;
} currency_t;



#ifdef _TESTING_
typedef struct stride_state {
	unsigned tickets;
	currency_state_t curr_state;
} stride_state_t;

typedef struct s_thread {
	stride_state_t		sched_state;
} s_thread_t;
#else
#include <fluke/thread.h>
#endif


/* 
 * Currency interface 
 */

/* Should be called before anything else */
void currency_init(currency_t *c, struct s_thread *t, unsigned base_tickets);

/* Should be called before inserting a thread in the tree */
void currency_state_init(struct s_thread *t);

#ifndef _TESTING_
/* Set the currency state */
void currency_set_state(currency_t *c, struct s_thread *target, 
			fluke_thread_state_t *state);

/* Get the currency state */
void currency_get_state(currency_t *c, struct s_thread *target, 
			fluke_thread_state_t *state);
#endif

/* Take a thread out of the tree */
void currency_thread_destroy(currency_t *c, struct s_thread *th);


#ifdef DEBUG
/* 
 *   Debug functions 
 */

/* Dump the currency_state inside the s_thread_t struct */
void currency_dump_state(struct s_thread *t);

/* Dump the hole currency tree */
void currency_dump_tree(currency_t *c);

/* Dump the subtree started at the provided node */
/* PRECOND: t must be <> NULL */
void currency_dump(struct s_thread *t);

/* Check the tree for inconsitencies */
int currency_check_tree(currency_t *c);
#endif

#endif













