/*
 * Copyright (c) 1995 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_KERNEL_DEBUG_H_
#define _FLUKE_KERNEL_DEBUG_H_

#include <oskit/debug.h>

extern int debug_indent;

#ifdef DEBUG
/* Symbol SOURCE should contain the name of the module */
#ifndef SOURCE
#define SOURCE "undef"
#endif

#define DEBUG_INDENT() {int i=0;for (;i<debug_indent;i++) printf("\t");}

#define DEBUG_INC_INDENT() indent++

#define DEBUG_DEC_INDENT() indent--

#define DEBUG_BASE_BEGIN() {\
	DEBUG_INDENT();\
	printf("DEBUG START:(%s) %s\n",SOURCE,__FUNCTION__);\
	debug_indent++;}

#define DEBUG_BASE_END() {\
	debug_indent--;\
	DEBUG_INDENT();\
	printf("DEBUG END  :(%s) %s\n",SOURCE,__FUNCTION__);}

#define DEBUG_BASE_BEGIN_ARGS(fmt, args...) {\
	DEBUG_INDENT();\
	printf("DEBUG START:(%s) %s(" fmt,SOURCE,__FUNCTION__,##args);\
	printf(")\n");\
	debug_indent++;}

#define DEBUG_BASE_END_ARGS(fmt, args...) {\
	debug_indent--;\
	DEBUG_INDENT();\
	printf("DEBUG END  :(%s) %s(" fmt,SOURCE,__FUNCTION__,##args);\
	printf(")\n");}

#define DEBUG_BASE()\
	DEBUG_INDENT();\
	printf("DEBUG      :(%s) %s\n",SOURCE,__FUNCTION__);

#define DEBUG_BASE_ARGS(fmt, args...){\
	DEBUG_INDENT();\
	printf("DEBUG      :(%s) %s(" fmt,SOURCE,__FUNCTION__,##args);\
	printf(")\n");}
#else
#define DEBUG_INDENT()
#define DEBUG_INC_INDENT()
#define DEBUG_DEC_INDENT()
#define DEBUG_BASE_BEGIN()
#define DEBUG_BASE_END()
#define DEBUG_BASE()
#define DEBUG_BASE_ARGS(fmt, args...)
#endif

#endif /* _FLUKE_KERNEL_DEBUG_H_ */
