/*
 * Copyright (c) 1996 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_KERNEL_KENTRY_H_
#define _FLUKE_KERNEL_KENTRY_H_

#include <oskit/machine/types.h>
#include <fluke/ob.h>

#include "link.h"

struct s_ob;
struct s_thread;
struct s_port;
struct fluke_thread_state;

/*
 * Change the specified user-space reference to contain the provided link.
 * Consumes the link on either success or failure.
 */
int kentry_ref_associate(
	struct s_thread		*cur_th,
	oskit_addr_t		ref_uva,
	struct link		*link);

/*
 * Look up the specified user-space reference, ensure it is of the correct
 * type, and return a copy of its link.
 */
int kentry_ref_extract_type(
	struct s_thread		*cur_th,
	oskit_addr_t		ref_uva,
	fluke_type_t		type,
	struct link		*out_link);

/*
 * Associate a reference object with an object of the indicated type.
 */
int kentry_ob_reference(
	struct s_thread		*thread,		
	oskit_addr_t		ob_uva,
	oskit_addr_t		ref_uva,
	fluke_type_t		type);

/*
 * Perform bootstrap initialization of the kentry layer.
 */
void kentry_boot(void);



#endif /* _FLUKE_KERNEL_KENTRY_H_ */
