/*
 * Copyright (c) 1997, 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * This is the header file for the kserver.
 * Public definitions for the Kernel Server layer,
 * which implements the minimal Common Protocols server
 * that services the root task's requests for memory, files, etc.
 */
#ifndef _FLUKE_KERNEL_KSERVER_H_
#define _FLUKE_KERNEL_KSERVER_H_

#if 0
#define DPRINTF(fmt, args...) \
	printf(fmt , ##args )
#define DMARK printf(__FUNCTION__": "__FILE__":%d: got here\n", __LINE__)
#else
#define DPRINTF(args...)	((void)0)
#define DMARK			((void)0)
#endif

#include "debug.h"	/* otsan */

#include <fluke/nest/server/server.h>
#include <fluke/nest/server/com.h>
#include <fluke/nest/nest_alloc.h>	/* nester memory allocation */
#include <oskit/page.h>	/* trunc_page, round_page, PAGE_SIZE */
#include <oskit/fs/dir.h>

#ifdef HEARTBEAT_THREAD
void *heartbeat_thread(void *);
#endif

/*
 * This is the granddaddy region that everything inherits from --
 * it is also accessable directly through /dev/kmem
 */
extern fluke_region_t kernel_region;

/*
 * Called in user mode by prog_start to start up the world.
 * Creates the kserver MOM classes and initial objects, ready to serve,
 * before loading and starting the first external user-level program.
 */
void kserver(char **argv, char **envp);

/*
 * Internal initializers for kserver modules; called by `kserver' (above).
 */
void kserver_mempool_init(void);
void kserver_console_init(void);
void kserver_clock_init(void);
void kserver_oskit_dev_init();	/* fdev drivers */
void ks_initialize_fdev();	/* fdev CP support */

#if FLASK
#include <oskit/com/sfs.h>

void kserver_avc_init(oskit_avc_t **avc);
oskit_error_t kserver_oskit_filepsid_init(oskit_dir_t *rootdir);

extern oskit_filepsid_t *kserver_oskit_filepsid;
#endif

/*
 * fluke::ttystream reference for the console device.
 * #ifndef CONFIG_FDEV, this is actually only a fluke::StreamIO object.
 */
extern mom_ref_t console_tty_ref;


/*
 * XXX: need to `fix' how we handle this
 */
extern size_t kserver_max_segsize;

/*
 */

extern fluke_task_t kernel_task; /*XXX in s_task.c */


fluke_ref_t *alloc_scratch_ref(void);
void free_scratch_ref(fluke_ref_t *ref);

/* Used for writes to the InitFDS */
void put_long_string(char *str, int size);

#ifdef MEMSTATS
extern unsigned long mstats_ks_bytes, mstats_ks_maxbytes;
extern unsigned long mstats_ks_ubytes, mstats_ks_maxubytes;
#endif

#if FLASK
#include "private_interfaces.h"
#endif 

#endif /* _FLUKE_KERNEL_KSERVER_H_ */
