/*
 * Copyright (c) 1997 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#include "list_static.h"
#include <stdio.h>
#include <stdlib.h>
#include "debug.h"
#include "kalloc.h"

void
list_create(list_t *l)
{
	l->size = 0;
	l->pos = 0;
	l->list = kalloc(MAX_LIST_SIZE * sizeof l->list[0]);
	assert(l->list);
}


void
list_destroy(list_t *l)
{
	assert(l->list);
	kfree(l->list, MAX_LIST_SIZE * sizeof l->list[0]);
}


/* Places window at begining of list */
void
list_go_first(list_t *l)
{
	l->pos = 0;
}

/* Returns 1 when window is at the end of the list */
int
list_end(list_t *l)
{
	return (l->pos == l->size);
}

/* Returns 1 when the list is empty */
int
list_empty(list_t *l)
{
	return (l->size == 0);
}

unsigned
list_size(list_t *l)
{
	return l->size;
}

/* Moves the window forward one elemente */
void
list_next(list_t *l)
{
	assert(l->pos < l->size);
	if (l->pos < l->size)
		l->pos++;
}

/* Gets element where the window is placed */
void *
list_get(list_t *l)
{
	return l->list[l->pos];
}

/* Inserts element after where the window is placed */
void
list_insert(list_t *l, void *e)
{
	int i;
	assert(l->size < MAX_LIST_SIZE);

	for (i = l->size; i > l->pos; i--)
		l->list[i] = l->list[i-1];
	l->list[l->pos] = e;
	l->size++;
}

void
list_remove(list_t *l)
{
	int i;
	assert(!list_empty(l));
	for (i = l->pos; i < l->size-1; i++)
		l->list[i] = l->list[i+1];
	l->size--;
}


/* Returns 1 is successful. Window left on the element */
int
list_find(list_t *l, void *e)
{
	for (l->pos = 0; l->pos < l->size; l->pos++)
		if (l->list[l->pos] == e)
			return 1;
	return 0;
}

/* Returns 1 is successful. Window left on the next element */
int
list_find_remove(list_t *l, void *e)
{
	int rc;
	if ((rc = list_find(l, e)))
		list_remove(l);
	return rc;
}
