/*
 * Copyright (c) 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * Table of static information about Fluke objects
 */
#include <stdlib.h>
#include "obinfo.h"

#include "s_ob.h"
#include "s_task.h"
#include "s_thread.h"
#include "s_mutex.h"
#include "s_cond.h"
#include "s_port.h"
#include "s_ref.h"
#include "s_mapping.h"
#include "s_region.h"


#include "config_opt_threads.h"
#ifdef CONFIG_OPT_THREADS
void s_thread_md_sanify_flob(struct s_ob *);
#else
#define s_thread_md_sanify_flob 0
#endif

#include "config_opt_synch.h"
#ifdef CONFIG_OPT_SYNCH
void s_mutex_md_sanify_flob(struct s_ob *);
#else
#define s_mutex_md_sanify_flob 0
#endif

#include "config_opt_ref.h"
#ifdef CONFIG_OPT_REF
void s_ref_sanify_flob(struct s_ob *);
#else
#define s_ref_sanify_flob 0
#endif

/* macro for table initializers, save a little typing */
#define FLOB(TYPE, type, s_align, u_sanify)				      \
[FLUKE_##TYPE] = { FLUKE_##TYPE##_SIZE, sizeof(s_##type##_t),		      \
		   s_align, u_sanify }


const struct obinfo obinfo[FLUKE_MI_MAX+1] = {
	FLOB(TASK,	task,	0, 		0),
	FLOB(THREAD,	thread,	S_THREAD_ALIGN,	s_thread_md_sanify_flob),
	FLOB(MUTEX,	mutex,	0, 		s_mutex_md_sanify_flob),
	FLOB(COND,	cond,	0, 		0),
	FLOB(PORT,	port,	0,		0),
	FLOB(PSET,	pset,	0,		0),
	FLOB(REF,	ref,	0,		s_ref_sanify_flob),
	FLOB(MAPPING,	mapping,0,		0),
	FLOB(REGION,	region,	0,		0),
};
