/*
 * Copyright (c) 1995-1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>

#include "s_thread.h"
#include "debug.h"
#undef dump_stack_trace

static int paniced;

#include "config_kmodule.h"

#ifndef CONFIG_KMODULE

#include <oskit/x86/proc_reg.h>
#include "prog.h"		/* XXX do_in_supervisor_mode */

struct args { const char *fmt; va_list vl; };
static int
do_panic(void *arg)
{
	struct args *const a = arg;

	vprintf(a->fmt, a->vl);
	printf("\n");
	paniced = 0;
	panic("panic in kserver user-mode thread");
	return KR_RESTART;
}
#endif


/*
 * Same as OSKit version with added thread state dump
 */
void panic(const char *fmt, ...)
{
	va_list vl;

	/* get the hell outta Dodge... */
	if (paniced)
		_exit(2);
	paniced = 1;

	va_start(vl, fmt);

#ifndef CONFIG_KMODULE
	if (get_cs() & 3) {
		struct args a = { fmt, vl };
		do_in_supervisor_mode(do_panic, &a);
	}
#endif

	vprintf(fmt, vl);
	va_end(vl);
	printf("\n");

#ifdef DEBUG
	s_debug_dump_flob(&current_thread()->sob, 0);
	printf("\nStack trace:\n");
	dump_stack_trace();
#endif

	/*
	 * Use _exit in case we were already in the process of exiting.
	 */
	_exit(1);
}
