/*
 * Copyright (c) 1997 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#include "prio_queue.h"
#include "debug.h"
#include <stdio.h>
#include "kalloc.h"

void
prio_queue_init(prio_queue_t *queue)
{
#ifndef NO_KERNEL
	spin_lock_init(&queue->lock);
#endif
	queue->head = &(queue->ghost);
	queue->ghost.next = NULL;
}

void
prio_queue_destroy(prio_queue_t *queue)
{
	prio_queue_node_t *tmp;

	while (!((tmp = queue->head->next) == NULL)) {
		queue->head->next = queue->head->next->next;
		kfree(tmp, sizeof *tmp);
	}

}

void
prio_queue_insert(prio_queue_t *q, prio_queue_node_t *elem, unsigned prio)
{
	prio_queue_node_t *prev;

	for (prev = q->head; prev->next != NULL && prio >= prev->next->prio;
	     prev = prev->next)
		;

	elem->next = prev->next;
	elem->prio = prio;
	prev->next = elem;
}


void
prio_queue_insert_2p(prio_queue_t *q, prio_queue_node_t *elem,
		     unsigned prio, unsigned prio2)
{
	prio_queue_node_t *prev;

	for (prev = q->head; prev->next != NULL && prio > prev->next->prio;
	     prev = prev->next)
		;
	for (; prev->next != NULL && prio == prev->next->prio &&
	     prio2 >= prev->next->prio2; prev = prev->next)
		;

	elem->next = prev->next;
	elem->prio = prio;
	elem->prio2 = prio2;
	prev->next = elem;
}



void
prio_queue_remove(prio_queue_t *q, prio_queue_node_t *elem)
{
	prio_queue_node_t *prev;

	for (prev = q->head; prev->next != NULL && prev->next != elem;
	     prev = prev->next)
		;
	if (prev->next != NULL)
		prev->next = elem->next;
	do_debug(elem->next = 0);
}


prio_queue_node_t *
prio_queue_remove_head(prio_queue_t *q)
{
	prio_queue_node_t *elem;

	elem = q->head->next;
	q->head->next = elem->next;
	do_debug(elem->next = 0);
	return elem;
}


prio_queue_node_t *
prio_queue_remove_head_prio(prio_queue_t *q, unsigned prio)
{
	prio_queue_node_t *prev;
	prio_queue_node_t *elem = 0;

	for (prev = q->head; prev->next != NULL && prio != prev->next->prio;
	     prev = prev->next)
		;
	if (prev->next != NULL) {
		elem = prev->next;
		prev->next = elem->next;
	}
	return elem;
}


int
prio_queue_find(prio_queue_t *q, prio_queue_node_t *elem)
{
	prio_queue_node_t *prev;

	for (prev = q->head; prev->next != NULL && prev->next != elem;
	     prev = prev->next)
		;
	return (prev->next != NULL);
}

int
prio_queue_empty(prio_queue_t *q)
{
	return (q->head->next == NULL);
}

prio_queue_node_t *
prio_queue_head(prio_queue_t *q)
{
	return q->head->next;
}

void
prio_queue_map(prio_queue_t *q, void f(prio_queue_node_t *))
{
	prio_queue_node_t *prev;

	for (prev = q->head; prev->next != NULL; prev = prev->next)
		f(prev->next);
}
