/*
 * Copyright (c) 1996 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * Contributed by the Computer Security Research division,
 * INFOSEC Research and Technology Office, NSA.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#if FLASK

/* FLASK */

#include "prog.h"		/* XXX for do_in_supervisor_mode */
#include "private_interfaces.h"
#include "exception.h"


int private_security_set(fluke_ref_t *ref)
{
    struct security_set_args args = { 
	(oskit_addr_t) ref }; 
    
    do_in_supervisor_mode(do_private_security_set, &args);
    return 0;    
}


int private_task_create_secure(fluke_task_t *task, security_id_t sid)
{
    struct task_create_secure_args args = { 
	(oskit_addr_t) task, 
	 sid }; 
    
    do_in_supervisor_mode(do_private_task_create_secure, &args);
    return 0;    

}

int private_segment_task_create(fluke_task_t *task, security_id_t sid)
{
    struct segment_task_create_args args = { 
	(oskit_addr_t) task, 
	 sid }; 
    
    do_in_supervisor_mode(do_private_segment_task_create, &args);
    return 0;    
}


int private_avc_reset(oskit_u32_t seqno)
{
    int exc_code = 0;
    struct avc_reset_args args = { seqno, &exc_code };
    
    do_in_supervisor_mode(do_private_avc_reset, &args);
    if (exc_code)
      return (exc_code);
    
    do_in_supervisor_mode(do_private_avc_stop_all_threads, 0);
    do_in_supervisor_mode(do_private_avc_reset_migrated, &args);
    do_in_supervisor_mode(do_private_avc_release_all_threads, 0);
    return exc_code;
}


int private_avc_remove(security_class_t sclass,
		       security_id_t ssid,
		       security_class_t tclass,
		       security_id_t tsid,
		       oskit_bool_t wired)
{
    int exc_code = 0;
    struct avc_remove_args args = { 
	sclass,	    
	 ssid,
	tclass,	    
	 tsid,
	wired,
        &exc_code };
    
    do_in_supervisor_mode(do_private_avc_remove, &args);
    return exc_code;
}    


int private_avc_try_remove(security_class_t sclass,
                           security_id_t ssid,
                           security_class_t tclass,
                           security_id_t tsid,
                           access_vector_t permissions,
                           oskit_bool_t removewired,
                           oskit_u32_t seqno,
                           access_vector_t *out_retained)
{
    int exc_code = 0;
    struct avc_try_remove_args args = {
        sclass,
        ssid,
        tclass,
        tsid,
        permissions,
        removewired,
        seqno,
	out_retained,
        &exc_code };
    
    do_in_supervisor_mode(do_private_avc_stop_all_threads, 0);
    do_in_supervisor_mode(do_private_avc_try_remove, &args);
    do_in_supervisor_mode(do_private_avc_release_all_threads, 0);
    return exc_code;
}


int private_avc_revoke(security_class_t sclass,
		       security_id_t ssid,
		       security_class_t tclass,
		       security_id_t tsid,
		       access_vector_t revoke,
		       oskit_bool_t wired,
		       oskit_u32_t seqno)
{
    int exc_code = 0;
    struct avc_revoke_args args = { 
	sclass,	    
	ssid,
	tclass,	    
	tsid,
	revoke,
	wired,
	seqno,
        &exc_code };
    
    do_in_supervisor_mode(do_private_avc_revoke, &args);
    if (exc_code)
        return exc_code;
    
    do_in_supervisor_mode(do_private_avc_stop_all_threads, 0);
    do_in_supervisor_mode(do_private_avc_revoke_migrated, &args);
    do_in_supervisor_mode(do_private_avc_release_all_threads, 0);
    return exc_code;
}    


int private_avc_add(security_class_t sclass,
		    security_id_t ssid,
		    security_class_t tclass,
		    security_id_t tsid,
		    access_vector_t allowed,
		    access_vector_t decided,
		    unsigned int expiration,
		    oskit_bool_t wired,	
		    oskit_bool_t replace,
		    oskit_u32_t seqno)
{
    int exc_code = 0;
    struct avc_add_args args = { 
	sclass,
	ssid,
	tclass,
	tsid,
	allowed,
	decided,
	expiration,
	wired,		
	replace,
	seqno,
        &exc_code };
    
    do_in_supervisor_mode(do_private_avc_add, &args);
    return exc_code;
}


int private_avc_has_permission(security_class_t sclass,
			       security_id_t ssid,
			       security_class_t tclass,
			       security_id_t tsid,
			       access_vector_t requested)
{
    int exc_code = 0;
    struct avc_has_permission_args args = { 
	sclass, 
	 ssid, 
	tclass, 
	 tsid, 
	requested,
        &exc_code };

    do_in_supervisor_mode(do_private_avc_has_permission, &args);
    return exc_code;
}

/* FLASK */

#endif
