/*
 * Copyright (c) 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * Contributed by the Computer Security Research division,
 * INFOSEC Research and Technology Office, NSA.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#if FLASK

#include "s_mapping.h"
#include "frag.h"
#include "debug.h"

#include "avc.h"

void
s_mapping_region_revoke(s_mapping_t * mapping, security_id_t tsid,
		    oskit_bool_t try_remove, access_vector_t * out_retained)
{
    int             rc;
    link_t          link;
    s_ob_t         *sob;


    link_init(&link);
    s_mapping_lock(mapping);

    s_mapping_f_lock(mapping);
    sob = link_follow(&mapping->region);
    s_mapping_f_unlock(mapping);

    if (sob && sob->sid == tsid) {
	s_ob_unlock(sob);
	if (!try_remove) {
	    rc = s_mapping_set_state(mapping, 0, &link, 0);
	    assert(rc == 0);
	} else {
	    *out_retained |= SUBJECT_SUBJECT__MAP;
	}
    } else if (sob)
	s_ob_unlock(sob);

    s_mapping_unlock(mapping);
}


int
s_mapping_region_reset(s_mapping_t * mapping)
{
    s_ob_t         *sob;
    link_t          link;
    oskit_bool_t    revoke;
    int             rc;


    link_init(&link);
    s_mapping_lock(mapping);

    s_mapping_f_lock(mapping);
    sob = link_follow(&mapping->region);
    s_mapping_f_unlock(mapping);

    if (sob) {
	rc = avc_has_permission_reset(SECCLASS_SUBJECT,
				      mapping->sob.sid,
				      SECCLASS_SUBJECT,
				      sob->sid,
				      SUBJECT_SUBJECT__MAP,
				      &revoke);
	s_ob_unlock(sob);
	if (rc) {
	    s_mapping_unlock(mapping);
	    return rc;
	}
	if (revoke) {
	    link_init(&link);
	    rc = s_mapping_set_state(mapping, 0, &link, 0);
	    assert(rc == 0);
	}
    }
    s_mapping_unlock(mapping);
    return 0;
}
#endif
