/*
 * Copyright (c) 1995 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLOSS_KERNEL_FLUKE_SUPER_PORT_H_
#define _FLOSS_KERNEL_FLUKE_SUPER_PORT_H_

#include <fluke/ipc.h>

#include "s_ob.h"

#if FLASK
#include <flask/security.h>
#endif 

typedef struct s_port
{
	struct s_ob		sob;

	/*
	 * Port set this port is attached to.
	 * If NULL, then the port is disabled;
	 * attempts to invoke it will return an error.
	 */
	struct link		pset_link;

	/*
	 * Alias value to pass back to the server
	 * when it receives an IPC invocation on this port.
	 */
	fluke_alias_t		alias;
} s_port;
typedef struct s_port s_port_t;

#define s_port_lock(port)	s_ob_lock(&(port)->sob)
#define s_port_unlock(port)	s_ob_unlock(&(port)->sob)

struct s_ref;
struct s_thread;
struct fluke_port_state;

int s_port_create(
	oskit_addr_t		uva,
	fluke_hash_t		hash,
	s_port_t		**out_port);

void s_port_destroy(
	s_port_t		*port);

int s_port_get_state(
	s_port_t		*port,
	struct fluke_port_state	*state,
	struct link		*pset_link);

int s_port_set_state(
	s_port_t		*port,
	struct fluke_port_state	*state,
	struct link		*pset_link);

/*
 * Attempt to find and capture a server thread on the port
 * specified by the client thread's IPC_CONNECT_DEST system call parameter.
 * On success (nonzero return), *out_server points to the captured thread.
 */
int s_port_capture(
	struct s_thread		*client,
#if FLASK
        access_vector_t		permission,			  
        security_id_t		client_sid,
#endif 
	struct s_thread		**out_server);
int s_port_capture_locked(
	struct s_port		*port,
	struct s_thread		**out_server);

#ifdef DEBUG
void s_port_dump(s_port_t *port, int indent);
#endif

#endif /* _FLOSS_KERNEL_FLUKE_SUPER_PORT_H_ */
