/*
 * Copyright (c) 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * Contributed by the Computer Security Research division,
 * INFOSEC Research and Technology Office, NSA.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#if FLASK

#include "s_thread.h"
#include "debug.h"

#include "avc.h"

void
s_thread_scheduler_revoke(s_thread_t * thread,
			  security_id_t tsid,
			  oskit_bool_t try_remove,
			  access_vector_t * out_retained)
{
#if defined(CONFIG_HIER_SCHED) && defined(CONFIG_HIER_SCHED_SET_PARENT)
    int             rc;
    s_ob_t         *sob;

    /* thread is locked and stopped on entry */

    sob = link_follow(&thread->sched_state.sched_link);

    if (sob && sob->sid == tsid) {
	s_ob_unlock(sob);
	if (!try_remove)
	    link_reinit(&thread->sched_state.sched_link);
	else
	    *out_retained |= SUBJECT_SUBJECT__THREAD_SCHEDULER;
    } else if (sob)
	s_ob_unlock(keeper);
#endif

    return;
}


int
s_thread_scheduler_reset(struct s_thread * thread, oskit_bool_t * out_reset)
{
#if defined(CONFIG_HIER_SCHED) && defined(CONFIG_HIER_SCHED_SET_PARENT)
    s_ob_t         *sob;
    oskit_bool_t    revoke;
    int             rc;
#endif


    *out_reset = FALSE;
#if defined(CONFIG_HIER_SCHED) && defined(CONFIG_HIER_SCHED_SET_PARENT)

    /* thread is locked and stopped on entry */

    sob = link_follow(&thread->sched_state.sched_link);

    if (sob) {
	rc = avc_has_permission_reset(SECCLASS_SUBJECT,
				      thread->sob.sid,
				      SECCLASS_SUBJECT,
				      sob->sid,
				      SUBJECT_SUBJECT__THREAD_SCHEDULER,
				      &revoke);
	s_ob_unlock(sob);
	if (rc) {
	    s_thread_unlock(thread);
	    return rc;
	}
	if (revoke) {
	    link_reinit(&thread->sched_state.sched_link);
	    *out_reset = TRUE;
	}
    }
#endif
    return 0;
}
#endif
