/*
 * Copyright (c) 1996, 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#include <oskit/x86/asm.h>
#include <fluke/x86/syscall.h>

#include "x86.h"
#include "s_thread_asm.h"
#include <fluke/exception.h>

/*
 * Special function used by functions
 * that need to be callable from either user or supervisor mode,
 * such as putchar(), smemalign(), etc.
 */
ENTRY(do_in_supervisor_mode)

	movl	4(%esp),%ecx
	movl	8(%esp),%edx

	/* If we're in user mode, make the magic system call */
	movw	%cs,%ax
	testb	$3,%al
	jnz	EXT(u_entry_page)+0xfe0 /* sys_special entrypoint */

	/* Already in supervisor mode; just call the function. */
	pushl	%edx
	call	*%ecx
	popl	%edx
	ret

ENTRY(sys_special)

	/* Find the current thread (ebx) */
	CURRENT_THREAD(%ebx)

	movl EXT(kernel_task_link), %eax
	cmpl %eax, TH_TASK(%ebx)
	jne 0f

	/* Find the function to be called and the argument to pass it */
	movl	TH_EXC_STATE+EXC_ECX(%ebx),%ecx
	movl	TH_EXC_STATE+EXC_EDX(%ebx),%edx

#ifdef DEBUG
	/* Link the caller into the backtrace */
	movl	TH_EXC_STATE+EXC_EBP(%ebx),%ebp
#endif

	/* Call the function */
	pushl	%edx
	call	*%ecx
	popl	%edx

	ret

	/* Bogus entry, not from a kserver thread */
0:	movl $0x7ff,TH_EXC_STATE+EXC_CODE(%ebx)
	movl $FLUKE_INSANITY_INVALID_ENTRYPOINT,TH_EXC_STATE+EXC_SUBCODE(%ebx)
	movl $KR_USER_EXCEPTION, %eax
	ret
