/*
 * Copyright (c) 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * This file defines the glue code used by Fluke API entry points
 * called from kernel mode.  In the kernel mode page tables (i.e.
 * above 0x80000000 where kernel-mode threads run), the Fluke entry
 * point area (0x8400nnnn) is mapped to the page defined in
 * kmodule_entry_page.S; each entry point looks like this:
 *
 *	pushl $sys_SYSCALL
 *	call *kmodule_entry_addr
 *
 * Because of the special mapping, that code must avoid PC-relative addressing
 * (the PC value will be 0x0400nnnn, but the linker will have resolved symbols
 * relative to the physical memory address of the kmodule_entry_page).
 * That is why it uses an indirect-absolute call through the constant
 * pointer we define below (kmodule_entry_addr).
 */

#include <oskit/x86/asm.h>
#include <fluke/x86/syscall.h>

#include "x86.h"
#include "s_thread_asm.h"


	.text
badkr:	.asciz "kmodule_entry: unexpected syscall KR_* value %x\n"

	P2ALIGN(3)
	.globl EXT(kmodule_entry_addr)
LEXT(kmodule_entry_addr)
	.long kmodule_entry

/*
 * On entry, the stack contains:
 *	0(esp)		->	entry point return address (0x0400nn0b)
 *	4(esp)		->	`sys_*' kernel syscall handler function
 *	8(esp) [user sp]->	original return address for syscall
 * All other registers are unchanged from the original call to the
 * Fluke entry point address; they carry the syscall arguments.
 */

ENTRY(kmodule_entry)

	/*
	 * Save %ebx and get &current_thread()->exc_state into %ebx.
	 */
	pushl %ebx
	CURRENT_THREAD(%ebx)
	addl $TH_EXC_STATE, %ebx

	/*
	 * Set the current thread's exc_state from the registers,
	 * as if it trapped when it jumped into the kernel entrypoint area.
	 */
	movl %edi, EXC_EDI(%ebx)
	movl %esi, EXC_ESI(%ebx)
	movl %ebp, EXC_EBP(%ebx)
	movl %edx, EXC_EDX(%ebx)
	movl %ecx, EXC_ECX(%ebx)
	movl %eax, EXC_EAX(%ebx)
	popl %eax		/* pop %ebx value saved above */
	movl %eax, EXC_EBX(%ebx)

	popl %eax		/* pop "kernel entry point" return value */
	popl %ecx		/* pop kernel syscall handler function */

	/*
	 * We have popped our return address into %eax; the sp is now back
	 * where it was on "kernel" entry.  We back up past the call that
	 * got us here (a 6-byte insn), and the pushl with the handler
	 * function address (a 5-byte insn), to point the thread's
	 * "exception PC" at the standard kernel entrypoint address for
	 * this syscall.
	 */
	subl	$11, %eax
	movl	%esp, EXC_ESP(%ebx)	/* syscalls examine this for args */
	movl	%eax, EXC_EIP(%ebx)

	/*
	 * Call the sys_* handler, as if from a user-mode system call.
	 */
	call	*%ecx
	orl	%eax,%eax	/* check return value for exception */
	jz	kmodule_success
0:	cmpl	$KR_RESTART, %eax
	je	kmodule_restart

	/*
	 * Handle the exception.
	 */
	pushl	%eax
	call	EXT(s_thread_exception)
	addl	$4,%esp
	orl	%eax,%eax	/* check return value for new exception */
	jz	kmodule_restart	/* zero return value means restart */
	jmp	0b		/* loop on KR_RESTART or new exception */

	ALIGN_TEXT
kmodule_success:
	/*
	 * Return to the fluke_nop entrypoint,
	 * causing the system call to "finish" instead of restart.
	 */
	movl	$fluke_nop__entry, EXC_EIP(%ebx)

kmodule_restart:

	/*
	 * Push the address to return to.
	 */
	movl	EXC_EIP(%ebx), %eax
	pushl	%eax

	/*
	 * Reset the registers from the exc_state.
	 */
	movl	EXC_EDI(%ebx), %edi
	movl	EXC_ESI(%ebx), %esi
	movl	EXC_EBP(%ebx), %ebp
	movl	EXC_EDX(%ebx), %edx
	movl	EXC_ECX(%ebx), %ecx
	movl	EXC_EAX(%ebx), %eax
	movl	EXC_EBX(%ebx), %ebx

	/*
	 * Return to the caller of the original entrypoint address.
	 */
	ret

/*
 * This is called from csw_start to get the first kmodule thread going.
 * The new (current) thread is the argument.
 */
ENTRY(kmodule_thread_start)
	movl	4(%esp), %ebx
	addl	$TH_EXC_STATE, %ebx
	movl	EXC_ESP(%ebx), %esp
	jmp	kmodule_restart
