/*
 * Copyright (c) 1996, 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * Simple kernel mutex implementation
 * for use higher-level kernel code.
 */

#ifndef _FLUKE_KERNEL_X86_MUTEX_MD_H_
#define _FLUKE_KERNEL_X86_MUTEX_MD_H_


/* we need to grab the machine independent mutex.... */
#include_next "mutex.h"

#ifdef CONFIG_KPREEMPT

#include "smp.h"

#include "config_opt_mutex.h"
#ifdef CONFIG_OPT_MUTEX

/*
 * Indicate that we implement these so the MI module doesn't
 * need to...
 */
#define MD_MUTEX_LOCK
#define MD_MUTEX_UNLOCK

#include "s_thread_md.h"	/* for current_thread() */
#include "preempt.h"

/*
 * Our fast mutex_lock
 *
 * We inline our mutex_lock with assembly to speed it up
 * The optimal path (when the mutex is not already locked) takes
 * exactly 12 cycles.
 *
 * XXX this will only work on 486 or higher processors.
 */
#define _mutex_lock(m) 	({						      \
	__label__ _locked;						      \
	struct mutex *const _m = (m);					      \
	struct s_thread *_owner;					      \
	asm volatile("
		cmpxchgl	%0,%1	# tests %2, %3 is %0
		jz	 	%4	# jump if owner was zero"	      \
		     : "=r" (_owner), "=m" (_m->owner)			      \
		     : "a" (0), "0" (current_thread()), "o" (*&&_locked)      \
		     : "memory");					      \
	disable_preemption();						      \
	pd_mutex_lock(current_thread(), _m);				      \
	enable_preemption();						      \
 _locked: })

/*
 * Our fast mutex_unlock...
 */
#define _mutex_unlock(m)  ({						      \
	struct mutex *const _m = (m);					      \
	struct s_thread *_th;						      \
	disable_preemption();						      \
	_th = wait_queue_remove_locked(&_m->queue);			      \
	_m->owner = _th;						      \
	if (_th)							      \
		thread_ready(_th);					      \
	enable_preemption(); })


#include "config_inlining.h"

#ifdef CONFIG_INLINING
#define mutex_lock(m)	_mutex_lock(m)
#define mutex_unlock(m)	_mutex_unlock(m)
#endif /* CONFIG_INLINING */

#endif /* CONFIG_OPT_MUTEX */
#endif /* CONFIG_KPREEMPT */
#endif /* _FLUKE_KERNEL_X86_MUTEX_MD_H_ */
