/*
 * Copyright (c) 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * This file defines the PC-lusring (that is, rollback/roll-forward)
 * that needs to be done for the various stages in the middle of the
 * u_ref_page.S calls that a user thread could be.
 */

#include "s_thread.h"
#include "s_ref.h"
#include "u_entry.h"
#include "exception.h"
#include <oskit/x86/eflags.h>


/*
 * %ecx may have been clobbered, but that's all.  Roll back.
 */
void
pclsr_u_ref_compare(struct s_thread *sth)
{
	EXC_SET_IP(sth, fluke_ref_compare__entry);
}

/*
 * The comparison has been done, we just need to return with the result.
 */
void
pclsr_u_ref_compare_cmpl(struct s_thread *sth)
{
	/* EFL_ZF is 0x40: result is 1 if that bit is set, 0 if not.  */
	EXC_SYSPARAM(sth, ref_compare, result) =
		((sth->exc_state.eflags & 0x40) >> 2);
	EXC_SET_IP(sth, fluke_nop__entry);
}


/*
 * The fetch has been done, clobbering the fluke_ref_t * argument.  The
 * final step of clearing the non-type bits from the handle may or may not
 * already have been done, but it doesn't hurt to repeat it here.
 */
void
pclsr_u_ref_type(struct s_thread *sth)
{
#if FLUKE_MI_MAX >= 0xf
#error need more type bits in u_ref handle
#endif
	EXC_SYSPARAM(sth, ref_type, result_type) &= 0xf;
	EXC_SET_IP(sth, fluke_nop__entry);
}
