/*
 * Copyright (c) 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#include <oskit/x86/asm.h>

#include "x86.h"
#include "idt.h"
#include "smp.h"

#define UENTRY_KPAGE	EXT(u_synch_page)
#define UENTRY_UPAGE	0x04001000
#include "u_entry.h"

/*
 * This file defines the 0x04001000 entrypoint page, which contains the
 * Fluke mutex and cond API calls.  We use user-mode code here for the
 * crucial mutex_trylock, mutex_lock, and mutex_unlock calls.  Their
 * atomicity is preserved by the rollback code in u_synch_pclsr.c (which see).
 */

	.text

	P2ALIGN(12)

	.globl	EXT(u_synch_page)
LEXT(u_synch_page)

/* 0x000 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_debug_dump_memuse)
FLUKE_ENTRY(sys_debug_null_syscall)
FLUKE_ENTRY(sys_debug_set_monregs)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* 0x100 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* 0x200 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* 0x300 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* 0x400 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* 0x500 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* 0x600 */
FLUKE_ENTRY(sys_mutex_create_hash)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

	/*
	 * The 486+ `cmpxchgl' instruction is:
	 *	cmpxchgl SRC,DEST
	 * it does (atomically):
	 *   if (%eax == DEST) {
	 *     xchgl SRC,DEST
	 *     cmpl SRC,DEST
	 *     set ZF=1
	 * } else {
	 *     set ZF=0
	 * }
	 */

/* 0x640 */
u_mutex_trylock:
	movl $1, %ecx		/* ecx = 1 = HELD */
	xorl %eax, %eax		/* eax = 0 = AVAILABLE */
	LOCKPREFIX
ANNO_UENTRY(u_mutex_trylock)
u_mutex_trylock_cmpxchg:
	cmpxchgl %ecx, UMX_STATE(%esi) /* if AVAILABLE, set to HELD */
	setnz %al		/* return 1 if not, 0 if so */
	ret
ANNO_UENTRY(u_mutex_trylock_cmpxchg)
9:	.space 0x40-(9b-u_mutex_trylock)


/* 0x680 */
u_mutex_lock:
	movl $1, %ecx		/* ecx = 1 = HELD */
	xorl %eax, %eax		/* eax = 0 = AVAILABLE */
	LOCKPREFIX
ANNO_UENTRY(u_mutex_lock)
u_mutex_lock_cmpxchg:
	cmpxchgl %ecx, UMX_STATE(%esi) /* if AVAILABLE, set to HELD */
	jnz 0f			/* was it? */
	ret			/* yes: we got the lock, return */
	ALIGN_TEXT
/*
 * The next entry slot at 0x690 is fluke_task_destroy, so we skip
 * one to the free slot at 0x6b0.  Because the syscall trap handler
 * always jumps into u_entry_page, we cannot conflict with other
 * entrypoint traps, even though we are only mapped in the 0x1000 page.
 */
9:	.space 0x30-(9b-u_mutex_lock)
/* 0x6b0 */
0:	FLUKE_ENTRY(sys_mutex_lock)
ANNO_UENTRY(u_mutex_lock_cmpxchg)
9:	.space 0x40-(9b-u_mutex_lock)

/* 0x6c0 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* 0x700 */
u_mutex_unlock:
	movl $1, %eax		/* eax = 1 = HELD */
	xorl %ecx, %ecx		/* ecx = 0 = AVAILABLE */
	LOCKPREFIX
u_mutex_unlock_cmpxchg:
ANNO_UENTRY(u_mutex_unlock)
	cmpxchgl %ecx, UMX_STATE(%esi) /* if HELD, set to AVAILABLE */
	jnz 0f			/* was it HELD? */
	ret			/* yes: nobody to wake up, all done, return */
/*
 * Nope, it was CONTESTED.  We must trap to the kernel to wake someone up.
 * 0x710 is fluke_task_reference, so skip one.
 */
9:	.space 0x20-(9b-u_mutex_unlock)
/* 0x720 */
0:	FLUKE_ENTRY(sys_mutex_unlock) /* trap to kernel to wake others */
ANNO_UENTRY(u_mutex_unlock_cmpxchg)
9:	.space 0x40-(9b-u_mutex_unlock)

/* 0x740 */
FLUKE_ENTRY(sys_mutex_destroy)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_mutex_get_state)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_mutex_move)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_mutex_set_state)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_mutex_reference)
FLUKE_ENTRY(sys_invalid)

/* 0x800 */
FLUKE_ENTRY(sys_cond_wait)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_cond_signal)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_cond_broadcast)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* 0x900 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_cond_create_hash)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_cond_destroy)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_cond_get_state)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_cond_set_state)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_cond_move)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_cond_reference)

/* 0xa00 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_port_create_hash)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_port_destroy)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_port_set_state)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_port_get_state)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_port_move)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_port_reference)

/* 0xb00 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* 0xc00 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_pset_create_hash)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_pset_destroy)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_pset_move)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_pset_reference)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* 0xd00 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_pset_set_state)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_pset_get_state)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* 0xe00 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* 0xf00 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
