/*
 * Copyright (c) 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * This file declares data structures used in the special gs page.
 */

#ifndef _FLUKE_KERNEL_X86_GSPAGE_H_
#define _FLUKE_KERNEL_X86_GSPAGE_H_

#include <fluke/thread.h>
#include <oskit/page.h>

/*
 * User-writable portion of the special page.
 */
struct gspage_u {
	/*
	 * It is a Good Thing for this to be 16-byte aligned.
	 */
	int scratch[4];
};

/*
 * Kernel-writable portion of the special page.
 * This is read-only in user mode.  Note that it's not really
 * protected from user writes, because it's the same one page.
 */
struct gspage_k {
	fluke_thread_t *cur_thread;
};

/*
 * This structure represents the page as seen through the GS segment.
 * The segment spans two virtual pages, the second of which is writable
 * only in kernel mode.
 */
struct gspage_seg {
	struct gspage_u u;
	/* -- page boundary -- */
	struct gspage_k k;
};

/*
 * This is the linear view of the whole single page.
 */
struct gspage_lin {
	struct gspage_k k;
	char space[(PAGE_SIZE - sizeof(struct gspage_k)
		    - sizeof(struct gspage_u))];
	struct gspage_u u;
};


#endif _FLUKE_KERNEL_X86_GSPAGE_H_
