/*
 * Copyright (c) 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * This file defines the pclsr_* C functions whose bodies are contained in
 * u_thread_page.S; see that file and x86/u_entry.h for an explanation of
 * what this is all about.
 */

#include "u_entry.h"

#ifdef ASSEMBLER
 #error no no no
#endif

#include "u_thread_page.S"


void
s_thread_md_sanify_flob(struct s_ob *sob)
{
	s_thread_t *const thread = (s_thread_t *)sob;

	s_thread_lock(thread);

	/*
	 * Clear unused bits just for tidiness.
	 */
	thread->th_lflags &= FLUKE_THREAD_EXCEPTION_ENABLE;

	/*
	 * The user could have clobbered the EXCEPTION_ENABLE bit
	 * without making the kernel notice.  We simulate a kosher
	 * fluke_thread_enable_exceptions syscall.
	 */
	if ((thread->th_lflags & FLUKE_THREAD_EXCEPTION_ENABLE) &&
	    (thread->gflags & FLUKE_THREAD_ASYNC_PENDING))
		thread_cancel(thread);

	/*
	 * Note that we don't care if they have clobbered their
	 * exception handlers.  The consequence is no worse than
	 * if they had set them to bogus values via thread_set_handlers.
	 */

	s_thread_unlock(thread);
}
