/*
 * Copyright (c) 1996-1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/* This contains the per-processor stuff */
#include "per_cpu.h"
#include <oskit/x86/base_paging.h>
#include "debug.h"
#include "pmap.h"
#include "gdt.h"

#include "intr.h"	/* assert_interrupts_disabled */
#include "kalloc.h"

#include "config_opt_threads.h"


#ifdef CONFIG_SMP
/*
 * Allocate and initialize the structure for a new CPU.
 * This is called only from the boot CPU.
 */
fluke_cpu_info_t *
per_cpu_init(int number)
{
	fluke_cpu_info_t *i;

	i = kalloc_aligned(sizeof *i, 4);
	assert(i);

	i->number = number;

	i->inval_tlb = 0;
	i->cancel_pending = 0;
#ifdef CONFIG_USER_SCHED
	i->yield_pending = 0;
#elif !defined(CONFIG_NOPREEMPT)
	i->reschedule = 0;
#endif
	i->preempters = 0;
#ifdef CONFIG_KPREEMPT
	i->preemption_enabled = 0;
#endif

	/* copy the default gdt and tss */
	assert(sizeof base_gdt == sizeof i->gdt);
	memcpy(&i->gdt, &base_gdt, sizeof base_gdt);
	i->tss = base_tss;
#ifdef IOPL0
	memset(i->bitmap, 0xff, sizeof(i->bitmap));
#endif

	/* first, update the gdt to point to MY tss... */
	/* adjust my TSS entry in GDT for the bitmap */
	fill_descriptor(&i->gdt[BASE_TSS >> 3],
#ifdef IOPL0
		kvtolin(&i->tss), sizeof(i->tss) + sizeof(i->bitmap) - 1,
#else
		kvtolin(&i->tss), sizeof(i->tss) - 1,
#endif
		ACC_PL_K | ACC_TSS, 0);

#ifdef CONFIG_OPT_THREADS
	/*
	 * Set up the per-processor special page.
	 */
	gspage_setup_cpu(i);
#endif

	return i;
}


/*
 * Load the stuff from per-cpu structure into the processor.
 */
void
per_cpu_load(fluke_cpu_info_t *i)
{
	struct pseudo_descriptor pdesc;

	/* Create a pseudo-descriptor describing the GDT.  */
	pdesc.limit = sizeof(i->gdt) - 1;
	pdesc.linear_base = kvtolin(&i->gdt);

	/* Load it into the CPU.  */
	set_gdt(&pdesc);

	/*
	 * I didn't change any of the segment registers, so I don't
	 * *have* to re-load all of them.
	 */
#if 1
	/* Load new seg regs */
	asm volatile("
		ljmp    %0,$1f
	1:
	" : : "i" (KERNEL_CS));
	set_ds(KERNEL_DS);
	set_es(KERNEL_DS);
	set_ss(KERNEL_DS);

	/*
	 * Clear out the FS and GS registers by default,
	 * since they're not needed for normal execution of GCC code.
	 */
	set_fs(0);
#ifdef CONFIG_OPT_THREADS
	set_gs(USER_GS);
#else
	set_gs(0);
#endif
#endif

	/* set up new idt */

	/* Create a pseudo-descriptor describing the IDT.  */
	pdesc.limit = sizeof(base_idt) - 1;
	pdesc.linear_base = kvtolin(&base_idt);

	/* Load the IDT.  */
	/* Should disable NMI as well... but if we get one we crash anyway. */
	{
		int flags = get_eflags();
		cli();
		set_idt(&pdesc);
		set_eflags(flags);
	}

	/* Now for the tss */
	/* it uses the values from the gdt, which DID change */
	/* Make sure the TSS isn't marked busy.  */
	i->gdt[BASE_TSS >> 3].access &= ~ACC_TSS_BUSY;

	/* Load the TSS.  */
	set_tr(BASE_TSS);

	/* base_critical_leave(); */
}
#endif
