/*
 * Copyright (c) 1997, 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#include <pthread.h>

#include "config_kmodule.h"

/*
 * XXX we really don't need/want it this big in general, but it has to
 * be large for any thread that calls the legacy device driver code.
 * XXX 8k was too small for using fdev freebsd/linux drivers;
 * 64k seems to be ok, maybe bigger than the drivers really need
 * XXX perhaps extend MOM infrastructure to tie a thread to an object
 * and use large stack only for threads specially tied to device objects;
 * another idea is to give MOM objects (or classes?) a required stack size,
 * and dynamically choose threads from the pool that have large enough
 * stacks, creating as needed.
 */
#define KTHREAD_STACK_SIZE (64*1024)


struct pthread {
	fluke_thread_t thread;	/* must come first */
	void **per_thread;
	int per_thread_size;
	int __errno;
#ifndef CONFIG_KMODULE		/* kmodule threads have only a kernel stack */
	void *stack;
	size_t stacksize;
#endif
};

pthread_attr_t kthread_attr_default;

extern fluke_mutex_t pthread_mutex;
extern fluke_cond_t pthread_cond;
extern struct pthread *killme;
extern int pthread_can_exit;

extern pthread_destructor_t per_thread_destr[PTHREAD_KEYS_MAX];

void pthread_uth_init(struct pthread *th); /* initialize per-thread data */

/* hacked per-thread entry point */
int pthread_setspecific_thread(struct pthread *thread, pthread_key_t key,
	const void *value);

extern struct link kernel_task_link; /* link for the kernel AS s_task */
