/*
 * Copyright (c) 1996-1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLUKE_KERNEL_X86_S_THREAD_H_
#define _FLUKE_KERNEL_X86_S_THREAD_H_

#include "config_nonblocking.h"
#include "config_opt_threads.h"


#ifdef CONFIG_NONBLOCKING
#define S_THREAD_SIZE		sizeof(struct s_thread)
#define S_THREAD_ALIGN		1 /* no special alignment requirement */
#else
#define S_THREAD_SHIFT		12
#define S_THREAD_SIZE		(1 << S_THREAD_SHIFT)
#define S_THREAD_ALIGN		S_THREAD_SHIFT
#endif


/* Per-thread machine-dependent thread layer state. */
struct s_thread_md
{
	/*
	 * Exception handler entrypoints.
	 */
	unsigned		trap_handler;
	unsigned		interrupt_handler;
	unsigned		client_alert_handler;
	unsigned		server_alert_handler;

	/*
	 * Exception state save area.
	 */
	unsigned		saved_eax;
	unsigned		saved_ecx;
	unsigned		saved_edx;
	unsigned		saved_eip;
	unsigned		saved_eflags;

#ifdef DEBUG
#define S_THREAD_SYSCALL_ST_SIZE	8
	unsigned		syscall_st[S_THREAD_SYSCALL_ST_SIZE];
#endif
};
#define th_saved_eax		md.saved_eax
#define th_saved_ecx		md.saved_ecx
#define th_saved_edx		md.saved_edx
#define th_saved_eip		md.saved_eip
#define th_saved_eflags		md.saved_eflags
#define th_trap_handler		md.trap_handler
#define th_interrupt_handler	md.interrupt_handler
#define th_client_alert_handler	md.client_alert_handler
#define th_server_alert_handler md.server_alert_handler

struct s_thread;
#ifdef DEBUG
void s_thread_store_stacktrace(struct s_thread *thread,
			       unsigned *tbuf, int tbsize);
void s_thread_print_stacktrace(struct s_thread *thread,
			       unsigned *tbuf, int tbsize);
#endif

#include <oskit/types.h>
register oskit_addr_t _sp_register_ __asm__("%esp");

#ifdef CONFIG_NONBLOCKING


/*
 * Each processor runs on a single naturally-aligned kernel stack,
 * and the current thread pointer is always kept
 * at offset zero in the stack area (the high end of the stack).
 * This means that if the stack overflows,
 * the current thread pointer is the first to go. :-)
 * This macro evaluates to an lvalue in the blocking kernel
 * so that it can be assigned to during context switches.
 */
#define current_thread()					\
	(*((struct s_thread**)(_sp_register_ & -PROCESSOR_STACK_SIZE)))

#else /* not CONFIG_NONBLOCKING */

/*
 * We know that the stack is in the thread,
 * and that the thread is naturally aligned;
 * therefore it's quite trivial to find the current thread.
 */
#define current_thread()					\
	((struct s_thread*)(_sp_register_ & -S_THREAD_SIZE))

#endif /* not CONFIG_NONBLOCKING */

#endif /* _FLUKE_KERNEL_X86_S_THREAD_H_ */
