/*
 * Copyright (c) 1996, 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * This file contains the basic glue code required to get SMP going
 * in Fluke.  It overwrites the default (do-nothing) paging primitives
 * in libsmp, and has the functions to get SMP initialized and started.
 */

/*
 * HACK  HACK  HACK  HACK
 */

#include "config_smp.h"

#ifdef CONFIG_SMP

#include <oskit/smp.h>
#include <oskit/x86/smp.h>
#include <oskit/x86/proc_reg.h>
#include <oskit/x86/base_cpu.h>
#include <oskit/x86/base_paging.h>
#include <oskit/x86/pc/phys_lmm.h>
#include <oskit/x86/eflags.h>
#include <oskit/debug.h>
#include "s_thread.h"
#include "per_cpu.h"
#include <fluke/fluke.h>	/* mom_set_exc */
#include "kalloc.h"

#include <fluke/machine/syscall.h>

#include <stdio.h>

/*
 * The SMP code calls this to map memory.  Since we use paging,
 * this actually does something.
 * Returns the kernel virtual address at which the mapping was made.
 */
oskit_addr_t smp_map_range(oskit_addr_t start_pa, oskit_size_t size)
{
	return pmap_map_phys_mem(start_pa, size, 1);
}


int smp_unmap_range(oskit_addr_t start, oskit_size_t size)
{
	printf("UNMapping 0x%x (%d) bytes at 0x%x\n", size, size, start);
	pdir_unmap_range(base_pdir_pa, start, size);
	return 0;
}



/*
 * This is called by main(), and it initializes the OS-Kit SMP support code
 */
void
smp_boot()
{
	smp_init();

	/*
	 * Store early the boot CPU's number, because assertions check
	 * for it agreeing with smp_find_cur_cpu().
	 */
	boot_cpu_info.number = smp_find_cur_cpu();

	/*
	 * Initialize the cpu chain as a proper circular list.
	 */
	boot_cpu_info.next = &boot_cpu_info;
}

static spin_lock_t thread_boot_lock = SPIN_LOCK_INITIALIZER;

/*
 * This is the first function the APs call in protected mode that
 * I know anything about.  They still need to be set up with their
 * own copies of things, but are running in protected mode with
 * basic mappings already.
 */
static void
run_ap(void *arg)
{
	fluke_cpu_info_t *const cpu = arg;
	s_thread_t *const cur_th = current_thread();

	base_paging_load();

	/*
	 * The BSP keeps this locked until it's switched off cur_th
	 * (actually, until it's taken cur_th->disp_state.lock).
	 */
	spin_lock(&thread_boot_lock);

	per_cpu_load(cpu);	/* Load up the magic bits.  */

	/*
	 * We won't get this lock until the BSP has switched away.
	 * Assign the CPU to this thread.
	 */
	spin_lock(&cur_th->disp_state.lock);
	cur_th->disp_state.cpu = cpu;
	spin_unlock(&cur_th->disp_state.lock);

	/* Enable receives of interrupts. */
	smp_message_pass_enable[cur_th->disp_state.cpu->number] = 1;

	/*
	 * Put our CPU on the list of all CPUs.
	 * At this point, we can get interrupted for rescheduling
	 * (except that interrupts are disabled).
	 */
	cur_th->disp_state.cpu->next = boot_cpu_info.next;
	boot_cpu_info.next = cur_th->disp_state.cpu;

	/*
	 * Our initial synchronization with the master CPU is complete,
	 * but it's off running other threads and has forgotten about us.
	 * We unlock this lock so it can use it again to boot the next CPU.
	 */
	spin_unlock(&thread_boot_lock);

	/* Set eflags & cr0 to reasonable values */
	sti();
	set_cr0(0x80000011);	/* XXX */
#ifndef IOPL0
	/* iopl = 3.  See x86/exception.c */
	set_eflags(get_eflags() | EFL_IOPL_USER);
#endif

#ifndef CONFIG_NONBLOCKING
	{
		/* XXX
		 * We need to reinitialize the csw state of
		 * our thread so csw_start will return to user mode.
		 */
		extern void thread_start(void);
		thread_disp_state_setup_stack(cur_th, thread_start,
					      &cur_th->exc_state);
	}
#endif /* CONFIG_NONBLOCKING */

	/*
	 * Start scheduling threads on this processor.
	 * Never returns.
	 */
	smp_disp_boot(cur_th);

	printf("Error: processor %d returned from smp_disp_boot\n",
		smp_find_cur_cpu());
	__asm__("hlt\n");
}

/*
 * Called via do_in_supervisor_mode.
 */
int
do_smp_start_cpu(void *arg)
{
	s_thread_t *cur_th = current_thread();
	mom_exc_t *const exc = arg;
	static int bsp, cpu = -1;

printf("do_smp_start_cpu on %p cpu %d\n",cur_th,smp_find_cur_cpu());
	if (bsp < 0) {		/* all CPUs have been started */
		mom_set_exc(exc, ECHILD);
		return 0;
	}

	if (cpu < 0) {		/* first time through */
		void apic_init(void); /* apic.c */

		apic_init();	/* alloc IPI handler */
		bsp = smp_find_cur_cpu();
		smp_message_pass_enable[bsp] = 1;

		assert(boot_cpu_info.number == bsp);

		spin_lock_init(&thread_boot_lock);
	}


#ifdef CONFIG_USER_SCHED
	spin_lock(&cur_th->disp_state.lock);
	if (cur_th->disp_state.provider == cur_th) {
		/*
		 * We were called from the root thread of another CPU!
		 * Switching this thread would leave that CPU in limbo.
		 */
		spin_unlock(&cur_th->disp_state.lock);
		mom_set_exc(exc, EBUSY);
		return 0;
	}
	spin_unlock(&cur_th->disp_state.lock);
#endif


	spin_lock(&thread_boot_lock);

	do			/* find the next cpu */
		cpu = smp_find_cpu(cpu);
	while (cpu == bsp);	/* skip the boot cpu */

	if (cpu == E_SMP_NO_PROC) { /* no CPUs left to start */
		bsp = -1;	/* leave a marker to future calls */
		spin_unlock(&thread_boot_lock);
		mom_set_exc(exc, ECHILD);
		return 0;
	}

	{
		fluke_cpu_info_t *info;
#ifdef CONFIG_NONBLOCKING
		/*
		 * Allocate the per-processor stack.
		 */
		char *sp = kalloc_aligned(PROCESSOR_STACK_SIZE,
					  PROCESSOR_STACK_SHIFT);
		if (!sp) {
			--cpu;
			spin_unlock(&thread_boot_lock);
			mom_set_exc(exc, ENOMEM);
			return 0;
		}
		sp += PROCESSOR_STACK_SIZE; /* stack grows down from top */
#else
		/*
		 * Run the new CPU lower down on the existing per-thread stack,
		 * leaving some slop below the part we will be using on this
		 * CPU before switching threads.
		 */
		void *sp = (char *) _sp_register_ - 256;
#endif

		/*
		 * Allocate a per-CPU data structure for this CPU.
		 * We can't change cur_th->disp_state.cpu while we're
		 * still running, so it's done by the AP itself.
		 */
		info = per_cpu_init(cpu);

		/*
		 * Start up the new CPU.
		 * We hold the kalloc_lock because smp_start_cpu
		 * lmm_free's the trampoline page.
		 */
		mutex_lock(&kalloc_lock);
		smp_start_cpu(cpu, run_ap, info, sp);
		mutex_unlock(&kalloc_lock);
	}


#ifdef DEBUG
	{
		char label[100];
		snprintf(label, sizeof(label), "root thread [CPU %d]", cpu);
		s_ob_lock(&cur_th->sob);
		s_ob_label(&cur_th->sob, label);
		s_ob_unlock(&cur_th->sob);
	}
#endif

	exc->_type = 0;		/* success indication to caller */
	exc->_except = (void *) cpu; /* pass back cpu number */

	/*
	 * Our user-mode state is in the sys_special entrypoint.
	 * "Finish" the syscall so do_in_supervisor_mode returns.
	 */
	EXC_SET_IP(cur_th, fluke_nop__entry);

	/*
	 * We hold the thread_boot_lock spin lock, which
	 * keeps the new CPU from starting before we switch away.
	 * Yield the original CPU to some other thread.
	 * Then the new CPU will return to user mode in the now-current thread.
	 */
	cur_th = pd_thread_dispatch(cur_th, &thread_boot_lock, 0, 0);

	/*
	 * We'd better be on the CPU assigned to the current thread!
	 */
	assert(smp_find_cur_cpu() == cur_th->disp_state.cpu->number);

#ifdef CONFIG_NONBLOCKING
	/*
	 * In the nonblocking model, we are still running on the BSP, on
	 * its per-processor stack.
	 */
	assert(cur_th->disp_state.cpu->number == bsp);
#else
	/*
	 * In the blocking model, we have just context-switched
	 * back into the original thread on the new CPU.
	 */
	assert(cur_th->disp_state.cpu->number != bsp);
#endif

	return KR_RESTART;
}

#ifndef CONFIG_USER_SCHED
#include <pthread.h>
#include "prog.h"


static void *cpu_boot(void *ignored) {
	mom_exc_t exc;
	printf("cpu_boot %p cpu %d\n",fluke_thread_self(),smp_find_cur_cpu());
	do_in_supervisor_mode(do_smp_start_cpu, &exc);
	assert(exc._type == 0);
	printf("CPU %d booted\n", (int) exc._except);
	return 0;
}

/*
 * This gets the additional processors running my function with their
 * own stack.
 * Once there, they can start scheduling threads and doing things.
 */
int
smp_start(void *ignored)
{
	int num_processors = smp_get_num_cpus();
	static int been_here_before;

	assert(!been_here_before);
	been_here_before++;

	while (--num_processors) {
		int rc;
		pthread_t uth;

		/*
		 * Create a kserver thread to be this processor's
		 * root thread.
		 */
		rc = pthread_create(&uth, NULL, cpu_boot, 0);
		assert_rc_0(rc);
		rc = pthread_detach(uth);
		assert_rc_0(rc);
	}

	return 0;
}

#endif /* ! CONFIG_USER_SCHED */

#endif /* CONFIG_SMP */
