/*
 * Copyright (c) 1996, 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Fluke Operating System.  The Fluke Operating System
 * is free software, also known as "open source;" you can redistribute it
 * and/or modify it under the terms of the GNU General Public License (GPL),
 * version 2, as published by the Free Software Foundation (FSF).  To explore
 * alternate licensing terms, contact the University of Utah at
 * csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The Fluke Operating System is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more
 * details.  You should have received a copy of the GPL along with the Fluke
 * Operating System; see the file COPYING.  If not, write to the FSF, 59 Temple
 * Place #330, Boston, MA 02111-1307, USA.
 */

#include <oskit/x86/asm.h>

#include "idt.h"

#include "config_profiling.h"
#include "config_user_gprof.h"
#include "config_opt_threads.h"
#include "config_opt_synch.h"
#include "config_opt_ref.h"

#include "k_entry_asm.h"

#ifdef CONFIG_OPT_THREADS
/* These system calls are implemented in u_thread_page. */
#define sys_thread_self				sys_invalid
/*#define sys_thread_enable_exceptions		sys_invalid*/
#define sys_thread_disable_exceptions		sys_invalid
#define sys_thread_get_saved_state		sys_invalid
#define sys_thread_set_saved_state		sys_invalid
/*#define sys_thread_return_from_exception	sys_invalid*/
#define sys_thread_get_handlers			sys_invalid
#define sys_thread_set_handlers			sys_invalid
#endif
#ifdef CONFIG_OPT_SYNCH
/* Some system calls are implemented in u_synch_page. */
#define OPT_SYNCH_ENTRY(normal, opt_synch)	FLUKE_ENTRY(opt_synch)
#else
#define OPT_SYNCH_ENTRY(normal, opt_synch)	FLUKE_ENTRY(normal)
#endif
#ifdef CONFIG_OPT_REF
/* Some system calls are implemented in u_ref_page. */
#define OPT_REF_ENTRY(normal, opt_ref)	FLUKE_ENTRY(opt_ref)
#else
#define OPT_REF_ENTRY(normal, opt_ref)	FLUKE_ENTRY(normal)
#endif

	.text

	P2ALIGN(12)

	.globl	EXT(u_entry_page)
LEXT(u_entry_page)

/* 0x000 */
#ifdef CONFIG_PROFILING
FLUKE_ENTRY(sys_doprof)
#else
FLUKE_ENTRY(sys_invalid)
#endif
FLUKE_ENTRY(sys_debug_puts)
FLUKE_ENTRY(sys_debug_dump_flob)
FLUKE_ENTRY(sys_debug_label_flob)
FLUKE_ENTRY(sys_thread_get_saved_state)
FLUKE_ENTRY(sys_debug_dump_memuse)
FLUKE_ENTRY(sys_debug_null_syscall)
FLUKE_ENTRY(sys_debug_set_monregs)
FLUKE_ENTRY(sys_ipc_send)
#if FLASK
FLUKE_ENTRY(sys_ipc_send_secure)
#else
FLUKE_ENTRY(sys_invalid)
#endif
#ifdef CONFIG_USER_GPROF
FLUKE_ENTRY(sys_profil)
#else
FLUKE_ENTRY(sys_invalid)
#endif
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_thread_set_saved_state)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* 0x100 */
FLUKE_ENTRY(sys_ipc_call)
#if FLASK
FLUKE_ENTRY(sys_ipc_call_secure)
#else
FLUKE_ENTRY(sys_invalid)
#endif
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_thread_return_from_exception)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_ipc_reply)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_thread_set_handlers)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_ipc_reply_wait_receive)
#if FLASK
FLUKE_ENTRY(sys_ipc_reply_wait_receive_secure)
#else
FLUKE_ENTRY(sys_invalid)
#endif
FLUKE_ENTRY(sys_ipc_client_connect_send)
#if FLASK
FLUKE_ENTRY(sys_ipc_client_connect_send_secure)
#else
FLUKE_ENTRY(sys_invalid)
#endif

/* 0x200 */
FLUKE_ENTRY(sys_thread_get_handlers)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_ipc_client_disconnect)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_thread_get_client)
#if FLASK
FLUKE_ENTRY(sys_thread_get_client_secure)
#else
FLUKE_ENTRY(sys_invalid)
#endif
FLUKE_ENTRY(sys_thread_set_client)
#if FLASK
FLUKE_ENTRY(sys_thread_set_client_secure)
#else
FLUKE_ENTRY(sys_invalid)
#endif
FLUKE_ENTRY(sys_thread_get_server)
#if FLASK
FLUKE_ENTRY(sys_thread_get_server_secure)
#else
FLUKE_ENTRY(sys_invalid)
#endif
FLUKE_ENTRY(sys_thread_set_server)
#if FLASK
FLUKE_ENTRY(sys_thread_set_server_secure)
#else
FLUKE_ENTRY(sys_invalid)
#endif

/* 0x300 */
FLUKE_ENTRY(sys_ipc_client_receive)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_thread_create_hash)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_thread_destroy)
FLUKE_ENTRY(sys_ipc_client_send)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_thread_set_state)
#if FLASK
FLUKE_ENTRY(sys_thread_set_state_secure)
#else
FLUKE_ENTRY(sys_invalid)
#endif
FLUKE_ENTRY(sys_thread_get_state)
#if FLASK
FLUKE_ENTRY(sys_thread_get_state_secure)
#else
FLUKE_ENTRY(sys_invalid)
#endif

/* 0x400 */
FLUKE_ENTRY(sys_thread_enable_exceptions)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_ipc_client_ack_send_over_receive)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_thread_disable_exceptions)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_ipc_client_send_over_receive)
FLUKE_ENTRY(sys_thread_interrupt)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_thread_self)
FLUKE_ENTRY(sys_invalid)

/* 0x500 */
FLUKE_ENTRY(sys_ipc_client_connect_send_over_receive)
FLUKE_ENTRY(sys_thread_move)
#if FLASK
FLUKE_ENTRY(sys_ipc_client_connect_send_over_receive_secure)
#else
FLUKE_ENTRY(sys_invalid)
#endif
FLUKE_ENTRY(sys_thread_reference)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_ipc_client_ack_send)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* 0x600 */
FLUKE_ENTRY(sys_mutex_create_hash)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_task_create_hash)
FLUKE_ENTRY(sys_invalid)
OPT_SYNCH_ENTRY(sys_mutex_trylock, sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_task_destroy)
FLUKE_ENTRY(sys_invalid)
OPT_SYNCH_ENTRY(sys_mutex_lock, sys_invalid) /* 0x680 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_task_set_state)
OPT_SYNCH_ENTRY(sys_invalid, sys_mutex_lock) /* 0x6b0: opt_synch kernel trap */
FLUKE_ENTRY(sys_task_get_state)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_task_move)
FLUKE_ENTRY(sys_invalid)

/* 0x700 */
OPT_SYNCH_ENTRY(sys_mutex_unlock, sys_invalid)
FLUKE_ENTRY(sys_task_reference)
OPT_SYNCH_ENTRY(sys_invalid, sys_mutex_unlock) /* 0x720: opt_synch trap */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_mutex_destroy)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_mutex_get_state)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_mutex_move)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_mutex_set_state)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_mutex_reference)
FLUKE_ENTRY(sys_invalid)

/* 0x800 */
FLUKE_ENTRY(sys_cond_wait)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_mapping_create_hash)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_mapping_destroy)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_cond_signal)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_mapping_set_state)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_cond_broadcast)
FLUKE_ENTRY(sys_mapping_get_state)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_mapping_move)

/* 0x900 */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_mapping_protect)
FLUKE_ENTRY(sys_cond_create_hash)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_cond_destroy)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_cond_get_state)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_cond_set_state)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_cond_move)
FLUKE_ENTRY(sys_ipc_wait_receive)
FLUKE_ENTRY(sys_cond_reference)

/* 0xa00 */
#if FLASK
FLUKE_ENTRY(sys_ipc_wait_receive_secure)
#else
FLUKE_ENTRY(sys_invalid)
#endif
FLUKE_ENTRY(sys_port_create_hash)
FLUKE_ENTRY(sys_ipc_server_disconnect)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_region_create_hash)
FLUKE_ENTRY(sys_port_destroy)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_region_destroy)
FLUKE_ENTRY(sys_port_set_state)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_port_get_state)
FLUKE_ENTRY(sys_region_set_state)
FLUKE_ENTRY(sys_port_move)
FLUKE_ENTRY(sys_region_get_state)
FLUKE_ENTRY(sys_port_reference)

/* 0xb00 */
FLUKE_ENTRY(sys_ipc_server_receive)
FLUKE_ENTRY(sys_region_move)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_region_reference)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_region_protect)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_ipc_server_send)
FLUKE_ENTRY(sys_region_search)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* 0xc00 */
FLUKE_ENTRY(sys_thread_yield)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_ipc_server_ack_send_over_receive)
FLUKE_ENTRY(sys_pset_create_hash)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_pset_destroy)
FLUKE_ENTRY(sys_ipc_server_send_over_receive)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_pset_move)
FLUKE_ENTRY(sys_ipc_server_send_wait_receive)
FLUKE_ENTRY(sys_pset_reference)
#if FLASK
FLUKE_ENTRY(sys_ipc_server_send_wait_receive_secure)
#else
FLUKE_ENTRY(sys_invalid)
#endif
FLUKE_ENTRY(sys_invalid)

/* 0xd00 */
FLUKE_ENTRY(sys_ipc_server_ack_send_wait_receive)
#if FLASK
FLUKE_ENTRY(sys_ipc_server_ack_send_wait_receive_secure)
#else
FLUKE_ENTRY(sys_invalid)
#endif
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_pset_set_state)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_pset_get_state)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_ipc_server_ack_send)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)

/* 0xe00 */
FLUKE_ENTRY(sys_instrument_thread) /* empty (valid) syscall ifndef CONFIG_INSTRUMENT */
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
#if FLASK
FLUKE_ENTRY(sys_security_reference)
#else
FLUKE_ENTRY(sys_invalid)
#endif
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_ipc_setup)
FLUKE_ENTRY(sys_invalid)

/* 0xf00 */
FLUKE_ENTRY(sys_ref_create)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_ref_destroy)
FLUKE_ENTRY(sys_invalid)
FLUKE_ENTRY(sys_invalid)
OPT_REF_ENTRY(sys_ref_hash, sys_invalid)
#if FLASK
FLUKE_ENTRY(sys_ref_sid)
#else
FLUKE_ENTRY(sys_invalid)
#endif
OPT_REF_ENTRY(sys_ref_type, sys_invalid)	
FLUKE_ENTRY(sys_ref_check)
FLUKE_ENTRY(sys_invalid)
OPT_REF_ENTRY(sys_ref_compare, sys_invalid)
FLUKE_ENTRY(sys_ref_copy)
FLUKE_ENTRY(sys_ref_move)
FLUKE_ENTRY(sys_special) /* 0xfe0: Kernel private, not in the Fluke API */

/* 0xff0: sys_nop (special) */
	ret
	.byte	0
	.short	0,0,0,0,0
	.long	EXT(sys_invalid)
